/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		ABSceneRunner.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/15	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	<iostream>
#include	"ABSceneRunner.h"
#include	"ABSRSceneBase.h"

//-------------------------------------------------------
/*
*	Constructor
*/
//-------------------------------------------------------
ABSceneRunner::ABSceneRunner()
:
	mVARunner( mVAResource ),
	m_sceneShadow( NULL )
{
}
//-------------------------------------------------------
/*
*	Destructor
*/
//-------------------------------------------------------
ABSceneRunner::~ABSceneRunner()
{
}
//-------------------------------------------------------
//	initialize
//-------------------------------------------------------
void	ABSceneRunner::initialize( void )
{
	this->clean();
}
//-------------------------------------------------------
//	fainalize
//-------------------------------------------------------
void	ABSceneRunner::finalize( void )
{
	this->clean();
}

//-------------------------------------------------------
//	scene control
//-------------------------------------------------------
//----------------------------
//	startScene
//----------------------------
bool	ABSceneRunner::startScene( void )
{
	ABint	i;
	ABbool	ret;

	if( this->m_sceneShadow ){
		ret = this->m_sceneShadow->onInitialize();
		if( ! ret ){ return false; }
	}

	for( i = 0 ; i < this->m_sceneArray.size() ; i ++ ){
		ret = this->eventCreateInstance( i );
		if( ! ret ){ return false; }
	};

	this->moveToScene( this->m_sceneNumberStart );
	return	this->controlScene();
}
//----------------------------
//	endScene
//----------------------------
void	ABSceneRunner::endScene( void )
{
	if( this->m_sceneNumberCurrent < this->m_sceneArray.size() ){
		if( this->m_sceneArray[this->m_sceneNumberCurrent].trueIsSceneActive ){
			this->eventExitScene( this->m_sceneNumberCurrent );
			this->m_sceneShadow->onExitScene();
		}
	}

	ABint	i;
	for( i = 0 ; i < this->m_sceneArray.size() ; i ++ ){
		this->eventReleaseInstance( i );
	};

	if( this->m_sceneShadow ){
		this->m_sceneShadow->onFinalize();
	}
}
//----------------------------
//	moveToScene
//----------------------------
bool	ABSceneRunner::moveToScene( ABuint32  inSceneNum )
{
	//	ɈړĂ̓_BSɌ݂̃V[̏܂ŒxȂ΂ȂȂB
	ABSRControl		controlStack;
	controlStack.control = ABSR_CONTROL_MOVE;
	controlStack.sceneNumber = inSceneNum;
	this->mControlStack.push_back( controlStack );
	
	return	true;
}

//----------------------------
//	controlScene
//----------------------------
bool	ABSceneRunner::controlScene( void )
{
	ABSRSceneBase*		sceneCurrent	= NULL;
	ABSRSceneBase*		sceneNext		= NULL;
	bool				result 			= true;

	while( ! this->mControlStack.empty() ){

		ABSRControl	 controlStack = this->mControlStack.front();
		this->mControlStack.pop_front();
	
		//	Scene̎̓X^[gƓ
		
		if( this->m_sceneShadow != NULL ){
			this->m_sceneShadow->onSpecificExitScene();
		}

		//	Sceneo
		if( this->m_sceneNumberCurrent != ABSR_SCENE_NOTHING ){	
			this->eventExitScene( this->m_sceneNumberCurrent );
		}

		this->m_sceneNumberCurrent = controlStack.sceneNumber;

		//	Scene֓
		if( this->m_sceneNumberCurrent != ABSR_SCENE_NOTHING ){
			this->eventEnterScene( this->m_sceneNumberCurrent );
		}
		
		if( this->m_sceneShadow != NULL ){
			//	񂾂Render
			this->eventGameLoop();
			this->m_sceneShadow->onSpecificEnterScene();
		}

		//	OK
		ABDebug( ABReportFormat( "Scene move to %d\n", this->m_sceneNumberCurrent ) );
	};
	
	return	result;
}

//----------------------------
//	get / set
//----------------------------
//-------------------------------------------------------
//	Get current shene number
//-------------------------------------------------------
ABuint32	ABSceneRunner::getCurrentSceneNumber( void )
{
	return	this->m_sceneNumberCurrent;
}

//-------------------------------------------------------
//	scene event
//-------------------------------------------------------
//	eventCreateInstance
bool	ABSceneRunner::eventCreateInstance( ABuint32  inSceneNum )
{
	this->m_sceneArray[inSceneNum].trueIsSceneActive = false;
	return	this->m_sceneArray[inSceneNum].sceneInstance->onInitialize();
}

//	eventReleaseInstance
void	ABSceneRunner::eventReleaseInstance( ABuint32  inSceneNum )
{
	if( this->m_sceneArray[inSceneNum].trueIsSceneActive ){
		this->eventExitScene( inSceneNum );
	}
	this->m_sceneArray[inSceneNum].sceneInstance->onFinalize();
}

//	eventEnterScene
void	ABSceneRunner::eventEnterScene( ABuint32  inSceneNum )
{
	this->m_sceneArray[inSceneNum].trueIsSceneActive = true;
	if( this->m_sceneShadow != NULL ){
		this->m_sceneShadow->onEnterScene();
	}

	this->m_sceneArray[inSceneNum].sceneInstance->onEnterScene();

}

//	eventExitScene
void	ABSceneRunner::eventExitScene( ABuint32  inSceneNum )
{
	this->m_sceneArray[inSceneNum].sceneInstance->onExitScene();
	if( this->m_sceneShadow != NULL ){
		this->m_sceneShadow->onExitScene();
	}
	this->m_sceneArray[inSceneNum].trueIsSceneActive = false;
}

//	eventGameLoop
void	ABSceneRunner::eventGameLoop( void )
{
	//	w߂s
	this->controlScene();

	if( this->m_sceneShadow != NULL ){
		this->m_sceneShadow->onEnterGameLoop();
	}
	
	this->m_sceneArray[ this->m_sceneNumberCurrent ].sceneInstance->onGameLoop();

	if( this->m_sceneShadow != NULL ){
		this->m_sceneShadow->onExitGameLoop();
	}
}

//	eventAppEvent
bool	ABSceneRunner::eventAppEvent( GEeventAppType  inAppEventType )
{
	bool	result = true;
	if( this->m_sceneShadow != NULL ){
		result = this->m_sceneShadow->onAppEvent( inAppEventType );
		if( ! result ){
			return	result;
		}
	}
	
	switch( inAppEventType ){
		case GE_EVENT_APP_QUIT:
			this->eventExitScene( this->m_sceneNumberCurrent );
			break;
		default:
			break;
	}
	
	return result;
}

//-------------------------------------------------------
//	scene
//-------------------------------------------------------
//	add scene
void	ABSceneRunner::addScene( ABSRSceneBase*  inScene, ABuint32  inSceneNum )
{
	
	if( this->m_sceneArray.size() <= inSceneNum ){
		this->m_sceneArray.resize( inSceneNum + 1 );
	}
	inScene->setSceneNumber( inSceneNum );
	this->m_sceneArray[ inSceneNum ].sceneInstance = inScene;
};

//	get scene
ABSRSceneBase*	ABSceneRunner::getScene( ABuint32  inSceneNum )
{
	if( this->m_sceneArray.size() < inSceneNum ){
		return	NULL;
	}
	return	this->m_sceneArray[inSceneNum].sceneInstance ;
};
ABSRSceneBase*	ABSceneRunner::getScene( void )
{
	return	this->m_sceneArray[this->m_sceneNumberCurrent].sceneInstance ;
};

//	get scene count
ABuint32	ABSceneRunner::getSceneCount( void )
{
	return	this->m_sceneArray.size();
}

//-------------------------------------------------------
//	shadow
//-------------------------------------------------------

//	set shadow
void	ABSceneRunner::setSceneShadow( ABSRSceneBase*  inSceneShadow )
{
	this->m_sceneShadow = inSceneShadow;
};
//	get shadow
ABSRSceneBase*	ABSceneRunner::getSceneShadow( void )
{
	return	this->m_sceneShadow;
};

//-------------------------------------------------------
//	start scene
//-------------------------------------------------------

//	set 
void	ABSceneRunner::setStartScene( ABuint32  inSceneNumber)
{
	this->m_sceneNumberStart = inSceneNumber;
}

//	get 
ABuint32	ABSceneRunner::getStartScene( void )
{
	return	this->m_sceneNumberStart;
}

//-------------------------------------------------------
//	clean
//-------------------------------------------------------
void	ABSceneRunner::clean( void )
{
	this->m_sceneArray.clear();
	this->m_sceneNumberCurrent	= ABSR_SCENE_NOTHING;
	this->m_sceneNumberStart	= ABSR_SCENE_NOTHING;
	this->m_sceneShadow			= NULL;
}

