/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		ABVAItemAnimation.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	"ABVAItemAnimation.h"
#include	"ABVAItemSprite.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS ABVAItemAnimationObject
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
ABVAItemAnimationObject::ABVAItemAnimationObject( void )
{
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
ABVAItemAnimationObject::~ABVAItemAnimationObject( void )
{
}

//-------------------------------------------------------------------------------------------------
//	setup
//-------------------------------------------------------------------------------------------------
ABbool	ABVAItemAnimationObject::setup( void )
{
	ABVAItemAnimation*		item;
	ABVAResource*			resource;
	ABVAItemObjectArray*	array;
	ABuint					instanceNumber = this->getInstanceNumber();

	this->mTargetSprite = NULL;
	this->mFrameNumber = 0;
	this->mFrameTime = 0;
	this->mPlayMode = M_STATE_STOP;
	this->mFrameCounter = 0;
	this->mNextSyncLocation = 0;
	this->mAnimationCount = 0;
	this->mDoBreakBeat = 0;

	//	^[QbgACeInstanceƐڑ
	ABVAGetResource( this, &item, &resource );
	ABTestReturnReportError( item->mTargetSprite == NULL, "item->mTargetSprite is null." );
	array = item->mTargetSprite->getInstances();	//	^[QbgACe̔z擾
	if( array->size() <= instanceNumber ){
		//	InstancêŏI
		ABTestReturnReportError( true, "instance count error." );
	}
	this->mTargetSprite = static_cast<ABVAItemSpriteObject*>( (*array)[instanceNumber] );
	this->mTargetSprite->mAnimationObject = this;
	this->mDoBreakBeat = false;

	//	Default
	if( item->mPlayMode == ABVAItemAnimation::M_PLAY_ONESHOT ){
		this->stop();
	}
	else{
		this->play();
	}

	return	true;
};

//-------------------------------------------------------------------------------------------------
//	cleanup
//-------------------------------------------------------------------------------------------------
void	ABVAItemAnimationObject::cleanup( void )
{
}

//-------------------------------------------------------------------------------------------------
//	ĐJn
//-------------------------------------------------------------------------------------------------
void	ABVAItemAnimationObject::play( void )
{
	ABVAItemAnimation*	animator = static_cast<ABVAItemAnimation*>(this->getArchitecture());

	this->mFrameTime	= animator->mFrameTimeMax;
	this->mNextSyncLocation = 0;
	
	if( ! animator->mTrueIsReversePlay ){
		this->mFrameNumber	= 0;
		this->mFrameCounter = 1;
		this->mPlayMode		= M_STATE_PLAY;
	}
	else{
		this->mFrameNumber	= animator->mFrameCountMax  - 1;
		this->mFrameCounter = -1;
		this->mPlayMode		= M_STATE_REVERSE;
	}
	++this->mAnimationCount;
}

//-------------------------------------------------------------------------------------------------
//	~
//-------------------------------------------------------------------------------------------------
void	ABVAItemAnimationObject::stop( void )
{
	this->mPlayMode		= M_STATE_STOP;

	ABVAItemAnimation*	animator = static_cast<ABVAItemAnimation*>(this->getArchitecture());
	switch( animator->mModule ){
		case ABVAItemAnimation::M_MODULE_NEON: 
			{
				ABVAItemSprite*		spriteItem = static_cast<ABVAItemSprite*>(this->mTargetSprite->getArchitecture());
				this->mTargetSprite->setCellNumber( M_MACBEAT_NEON_GRAY, -1 );
				this->mTargetSprite->mSprite.m_color.set( 255, 255, 255, 255 );
				this->mTargetSprite->mSprite.update();
			}
			break;
	}
}

//-------------------------------------------------------------------------------------------------
//	ĐH
//-------------------------------------------------------------------------------------------------
ABbool	ABVAItemAnimationObject::isPlay( void )
{
	bool	flag = false;

	switch( this->mPlayMode ){
		case M_STATE_PLAY:
			flag = true;
			break;
		case M_STATE_REVERSE:
			flag = true;
			break;
		case M_STATE_STOP:
		default:
			break;
	}

	return	flag;
}

//-------------------------------------------------------------------------------------------------
//	Aj[Vs
//-------------------------------------------------------------------------------------------------
void	ABVAItemAnimationObject::animate( ABVAItemAnimation*  inAnimator,  ABint  inBMSLocation, ABbool  inDoBreakBeat )
{
	ABint  inNowEventLocation = inBMSLocation;
	if( ! this->mDoBreakBeat ){
		//	truêƂ͂܂uCNĂȂ̂ŁAȂB
		this->mDoBreakBeat = inDoBreakBeat;
	}

	++inNowEventLocation;
	
	if( this->mPlayMode == M_STATE_STOP ){
		return;
	}
	
	switch( inAnimator->mSyncMode ){

		case ABVAItemAnimation::M_SYNC_BPM:

			//	BPMɓcountdown
			if( this->mNextSyncLocation <= inNowEventLocation  ){	

				this->countDown( inAnimator );

				//	̓^C~OZo
				//	mBPMSyncRate؂̂āAZB
//				this->mNextSyncLocation = this->mNextSyncLocation - ( inNowEventLocation % inAnimator->mBPMSyncRate );
				this->mNextSyncLocation = inNowEventLocation - (inNowEventLocation % inAnimator->mBPMSyncRate);
				this->mNextSyncLocation += inAnimator->mBPMSyncRate;
			}
			break;
			
		case ABVAItemAnimation::M_SYNC_NORMAL: 
		default:
			//	lɖcountdown
			this->countDown( inAnimator );
			break;
	}

}

//-------------------------------------------------------------------------------------------------
//	countdown
//-------------------------------------------------------------------------------------------------
void	ABVAItemAnimationObject::countDown( ABVAItemAnimation*  inAnimator )
{
	ABVAItemAnimation*		animator = inAnimator;
	bool					flagReset = false;

	//	count down
	--this->mFrameTime;

	//	check
	if( this->mFrameTime < 0 ){
		//	reset
		this->mFrameTime = animator->mFrameTimeMax;

		//	go to next frame
		this->mFrameNumber += this->mFrameCounter;
		
		//	͈check
		if( this->mFrameNumber < 0 ){
			flagReset = true;
		}
		else if( animator->mFrameCountMax <= this->mFrameNumber ){
			flagReset = true;
		}
		//	ZbgH
		if( flagReset ){
			//	never ? one shot?
			switch( animator->mPlayMode ){
				case	ABVAItemAnimation::M_PLAY_ONESHOT:
					//	animtaion stop
					this->stop();
					return;//	̂Ƃ͑₩ɏIB
				case	ABVAItemAnimation::M_PLAY_NEVER:
					//	restart animation
					this->play();
				default:
					break;
			}
		}
		
		ABint	frameNumber = this->mFrameNumber;
		
		//	u[Ngp邩H
		if( this->mDoBreakBeat ){
			//	t[ԍu
			//	̌ʂ͎̃t[\܂ňێB
			if( animator->mBreakBeatCellNumber == -1 ){
				//	Ō̃t[̎̃t[摜gpB
				frameNumber = animator->mFrameCountMax;
			}
			else{
				//	w̃t[gp
				frameNumber = animator->mBreakBeatCellNumber;
			}
			this->mDoBreakBeat = false;
		}
		
		//	Animation@
		switch( animator->mModule ){
			case ABVAItemAnimation::M_MODULE_NORMAL:
				//	setting cell number
				if( animator->mTrueIsCellAnimation ){
					switch( animator->mSurfaceLayoutType ){
						case ABVAItemAnimation::M_LAYOUT_HORIZON:
							this->mTargetSprite->setCellNumber( frameNumber, -1 );
							break;
							
						case ABVAItemAnimation::M_LAYOUT_VERTICAL: 
						default:
							this->mTargetSprite->setCellNumber( -1, frameNumber );
							break;
					}
					this->mTargetSprite->mSprite.update();
				}
				break;

			case ABVAItemAnimation::M_MODULE_NEON: 
				break;
		}
	}

	//	Animation
	switch( inAnimator->mModule ){
		case ABVAItemAnimation::M_MODULE_NEON: 
			{
				static	ABuint			rondomNumberCounter = 0;			// <-- ȂyK

				rondomNumberCounter = rondomNumberCounter * 5 + 2 ;
				ABuint	rondomNumber = rondomNumberCounter & 255;

				if( inAnimator->mTrueIsCellAnimation ){

					ABVAItemSprite*		spriteItem = static_cast<ABVAItemSprite*>(this->mTargetSprite->getArchitecture());

					switch( inAnimator->mSurfaceLayoutType ){
						case ABVAItemAnimation::M_LAYOUT_NEON:
						default:

							//	vO~OIAnimation
							if( (this->mFrameTime % 7) == 0){

								//	Z؂ւ
								//	spriteItem->mCellNumber.xOŐ؂ւƁAjudgeɉēK؂ȉ摜Ił܂B
								if( (this->mFrameTime % 2) == 0){
									this->mTargetSprite->setCellNumber( spriteItem->mCellNumber.x, -1 );
								}
								else{
									this->mTargetSprite->setCellNumber( spriteItem->mCellNumber.x + M_MACBEAT_NEON_DARK_REACH, -1);
								}

							}
							else if( (this->mFrameTime % 3) == 0){
								//	_ȃp^[alphauhύX
								//	ȂƂȁ[ۂȂ܂I
								this->mTargetSprite->mSprite.m_color.set( 255, 255, 255, 255 - 127 + ( rondomNumber & 127 ) );
							}
							break;

					}
					this->mTargetSprite->mSprite.update();
				}

			}
			break;
	}

}

//-------------------------------------------------------------------------------------------------
//	Instancẽf[^ݒ肷
//-------------------------------------------------------------------------------------------------
ABbool	ABVAItemAnimationObject::setData( const ABVAItemObjData  inType, void*  outData )
{
	return	true;
}

//-------------------------------------------------------------------------------------------------
//	Instancẽf[^擾
//-------------------------------------------------------------------------------------------------
ABbool	ABVAItemAnimationObject::getData( const ABVAItemObjData  inType, void**  outData )
{
	return	true;
}
