/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		ABVAItemTextSprite.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	"ABVAItemTextSprite.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS ABVAItemTextSpriteObject
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
ABVAItemTextSpriteObject::ABVAItemTextSpriteObject( void )
{
	mText = NULL;
	mTextSprite = NULL;
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
ABVAItemTextSpriteObject::~ABVAItemTextSpriteObject( void )
{
}

//-------------------------------------------------------------------------------------------------
//	setup
//-------------------------------------------------------------------------------------------------
ABbool	ABVAItemTextSpriteObject::setup( void )
{
	ABbool					ret;
	ABVAItemTextSprite*		item;
	ABVAResource*			resource;
	ABVAItemResImageObject*		imageObject = NULL;
	ABVAItemResFontObject*		fontObject = NULL;

	ABVAGetResource( this, &item, &resource );

	try{
		if( item->mUseFont ){
			this->mTextSprite	= new GGTextSprite;
		}
		else{
			this->mText			= new GGText;
		}
	}
	catch( ... ){
		return false;
	}

	if( item->mUseFont ){
		ABVAItemResFont*	font = item->mFont[ item->mFontTableNumber ];

		this->mTextSprite->mWidth			= item->mWidth;
		this->mTextSprite->mHeight			= item->mHeight;
		this->mTextSprite->mLocate			= item->mLocation;
		this->mTextSprite->mColor			= item->mColor;

		this->mTextSprite->mTextLengthMax	= item->mTextMaxLength;

		fontObject = static_cast<ABVAItemResFontObject*>( font->getFisrstInstance() );
		this->mTextSprite->mFont			= &fontObject->mFont;

		if( item->mSurface ){
			imageObject = static_cast<ABVAItemResImageObject*>( item->mSurface->getFisrstInstance() );
			this->mTextSprite->mSurface = &imageObject->mTexture;
		}
		else{
			return false;
		}
		
		//	tHgɃXvCg𒲐
		this->mTextSprite->mWidth	= font->mCellWidth;
		this->mTextSprite->mHeight	= font->mCellHeight;
		
		
		ret = this->mTextSprite->setup(  resource->mRenderGroup[ 0 ], false );
		if( ! ret ){ return ret; }
		ret = this->mTextSprite->draw( const_cast<char*>(item->mString.c_str()) );
		if( ! ret ){ return ret; }
	}
	else{
		this->mText->mSprite.m_width	= item->mWidth;
		this->mText->mSprite.m_height	= item->mHeight;
		this->mText->mSprite.m_locate	= item->mLocation; 
		this->mText->mSprite.m_color	= item->mColor;
		this->mText->mSprite.setup(  resource->mRenderGroup[ 0 ], false );
		this->mText->mFontName			= item->mFontName;
		this->mText->mFontSize			= item->mFontSize;
		this->mText->mAlignX			= item->m_textAlignX;
		this->mText->mAlignY			= item->m_textAlignY;
		this->mText->mUntiAliasing	= true;
		this->mText->mFontStyle		= GE_FONT_NOMAL;
		this->mText->mUseBackColor	= item->mUseBackColor;
		this->mText->mBackColor		= item->mBackColor;
		this->mText->mSoftReturn	= item->mSoftReturn;
		ret = this->mText->setup();
		if( ! ret ){ return ret; }
		ret = this->mText->draw( item->mText.get() );
		if( ! ret ){ return ret; }
	}

	return	true;
};

//-------------------------------------------------------------------------------------------------
//	cleanup
//-------------------------------------------------------------------------------------------------
void	ABVAItemTextSpriteObject::cleanup( void )
{
	if( this->mText ){
		delete	this->mText;
	}
	if( this->mTextSprite ){
		delete	this->mTextSprite;
	}
}

//-------------------------------------------------------------------------------------------------
//	Instancẽf[^ݒ肷
//-------------------------------------------------------------------------------------------------
ABbool	ABVAItemTextSpriteObject::setData( const ABVAItemObjData  inType, void*  outData )
{
	return	true;
}

//-------------------------------------------------------------------------------------------------
//	Instancẽf[^擾
//-------------------------------------------------------------------------------------------------
ABbool	ABVAItemTextSpriteObject::getData( const ABVAItemObjData  inType, void**  outData )
{
	return	true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS ABVAItemTextSprite
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
//	dynamic bind render
//-------------------------------------------------------------------------------------------------
void	ABVAItemTextSprite::dynamicBindRender( ABVAItemDynamicBinder*  inBinder )
{
	if( inBinder->mBindData == NULL ){
		//	󂯕tȂ
		return ;
	}
	if( inBinder->mMode != ABVA_ITEM_DYNAMIC_BIND_TEXT ){
		//	󂯕tȂ
		return ;
	}
	
	ABVAItemObjectArray*		array = this->getInstances();
	ABVAItemTextSpriteObject*	instance = static_cast<ABVAItemTextSpriteObject*>( (*array)[ 0 ] );
	
	if( this->mUseFont ){
		if( inBinder->mBindData->mValuePointer ){
			instance->mTextSprite->draw( static_cast<ABint8*>( inBinder->mBindData->mValuePointer ) );
		}
		else{
			instance->mTextSprite->draw( "0" );
		}
	}
	else{
		GameString	text;
		if( inBinder->mBindData->mValuePointer ){
			ABint8*		cs = static_cast<ABint8*>( inBinder->mBindData->mValuePointer );
			if( cs[0] != 0 ){
				text.setCString( cs, GS_ENCORDING_SHIT_JIS );
				instance->mText->draw( text.get() );
				cs[0] = 0;
			}
		}
		else{
			text.setCString( "NULL" );
			instance->mText->draw( text.get() );
		}
	}
}
