/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		ABVARunner.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////
#define		ABVA_RUNNER_CPP

#include	"ABVARunner.h"
#include	"ABVAItemPreset.h"
#include	"ABVAItem.h"
#include	"ABVAItemObject.h"
#include	"ABVAItemBinder.h"
#include	"ABVAItemRender.h"

#include	"GUIView.h"
#include	"GameSingleton.h"
#include	"ABSingleton.h"
#include	"AquanBeatsPreference.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS	ABVARunner
//
//////////////////////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
ABVARunner::ABVARunner( ABVAResource&  inResource )
:
	m_currentList( NULL ),
	m_currentItemArray( NULL ),
	m_resource( inResource ),
	m_currentBinder( NULL ),
	m_currentRenderer( NULL ),
	m_bmsLocation( 0 ),
	m_screenWidth( 640 ),
	m_screenHeight( 480 )
{
	memset( m_binderArray, NULL, sizeof(m_binderArray) );
	memset( m_renderer, NULL, sizeof(m_renderer) );
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
ABVARunner::~ABVARunner( void )
{
	this->releaseItems();
}

//-------------------------------------------------------------------------------------------------
//	ACe쐬
//-------------------------------------------------------------------------------------------------
ABbool	ABVARunner::createItems( ABVAItemPreset&  inPreset )
{
	ABVASchemeSetupper		setupper( *this );
	
	this->releaseItems();
	
	try{
		inPreset.setup( setupper );
	}
	catch( std::bad_alloc ){
		return false;
	}
	return true;
}

//-------------------------------------------------------------------------------------------------
//	ACep
//-------------------------------------------------------------------------------------------------
void	ABVARunner::releaseItems( void )
{
	ABint	i;

	ABVAItemListIt		itemIt;

	for( i = 0 ; i < ABVA_ITEM_LIST_MAX ; i ++ ){
	
		while( ! this->m_itemList[i].empty() ){
			ABVAItem*	item = this->m_itemList[i].back();

			delete item;

			this->m_itemList[i].pop_back();
		}

		this->m_itemArray[ i ].clear();
		
		delete	this->m_binderArray[ i ];
		this->m_binderArray[ i ] = NULL;
		delete	this->m_renderer[ i ];
		this->m_renderer[ i ] = NULL;
	}
	this->m_currentList		= NULL;
	this->m_currentItemArray= NULL;
}

//-------------------------------------------------------------------------------------------------
//	Instance쐬
//-------------------------------------------------------------------------------------------------
ABbool	ABVARunner::createInstances( ABuint  inListNumber )
{
	ABbool				ret = false;
	ABVAItemListIt		itemIt;

	if( ABVA_ITEM_LIST_MAX < inListNumber ){
		return false;
	}

	this->m_currentList			= &this->m_itemList[ inListNumber ];
	this->m_currentItemArray	= &this->m_itemArray[ inListNumber ];
	this->m_currentBinder		= this->m_binderArray[inListNumber];
	this->m_currentRenderer		= this->m_renderer[inListNumber];

	if( this->m_currentList->empty() ){
		ABDebug( ABReport( "empty item list." ) );
		return true;
	}

	for( itemIt = this->m_currentList->begin(); itemIt != this->m_currentList->end(); itemIt++ ){
		ret = (*itemIt)->create( this );
		ABTestBreakReportError( ! ret, "create." );
	}
	if( ! ret ){
		return ret;
	}

	if( this->m_currentRenderer ){
		this->m_currentBinder->initialize();
		this->m_currentRenderer->initialize( this, this->m_currentBinder );
	}

	return ret;
}

//-------------------------------------------------------------------------------------------------
//	insutancep
//-------------------------------------------------------------------------------------------------
void	ABVARunner::releaseInstances( void )
{
	ABVAItemListIt		itemIt;
	
	if( this->m_currentList == NULL ){
		return;
	}
	
	if( this->m_currentRenderer ){
		this->m_currentRenderer->finalize( this, this->m_currentBinder );
	}

	for( itemIt = this->m_currentList->begin(); itemIt != this->m_currentList->end(); itemIt++ ){
		ABVAItem*	item = *itemIt;
		item->destroy();
	}
}

//-------------------------------------------------------------------------------------------------
//	ACe擾
//-------------------------------------------------------------------------------------------------
ABVAItem*		ABVARunner::getItem( ABuint  inItemNumber )
{
	if( this->m_currentItemArray->size() <= inItemNumber ){
		return NULL;
	}
	ABVAItemListIt	it = (*this->m_currentItemArray)[inItemNumber];
	return	*it;
}

//-------------------------------------------------------------------------------------------------
//	ACeLInstance擾
//-------------------------------------------------------------------------------------------------
ABuint			ABVARunner::getInstanceCount( ABuint  inItemNumber )
{
	ABVAItem*	item = this->getItem( inItemNumber );
	if( item == NULL ){
		return 0;
	}
	ABVAItemObjectArray*	array = item->getInstances();
	return	array->size();
}

//-------------------------------------------------------------------------------------------------
//	ACe琶ꂽInstance擾
//-------------------------------------------------------------------------------------------------
ABVAItemObject*	ABVARunner::getInstance( ABuint  inItemNumber, ABuint  inInstanceNumber )
{
	ABVAItem*	item = this->getItem( inItemNumber );
	if( item == NULL ){
		return NULL;
	}
	ABVAItemObjectArray*	array = item->getInstances();
	if( array->size() <= inInstanceNumber ){
		return NULL;
	}
	return	(*array)[inInstanceNumber];
}

//-------------------------------------------------------------------------------------------------
//	InstanceRender
//-------------------------------------------------------------------------------------------------
void	ABVARunner::render( void )
{
	if( this->m_currentList == NULL ){
		return;
	}

	//	RenderNX̃_O
	if( this->m_currentRenderer ){
		this->m_currentRenderer->render( this, this->m_currentBinder );
	}

	//	eACe̓ꃌ_O
	ABVAItemListIt		itemIt;
	for( itemIt = this->m_currentList->begin(); itemIt != this->m_currentList->end(); itemIt++ ){
		(*itemIt)->callRender( this->m_bmsLocation );
	}
}

//-------------------------------------------------------------------------------------------------
//	Q[XN[TCY
//	r[TCYƂ͕ʕłBSchemeɂĎw肳A`Ԃ̉zIȃTCYłB
//-------------------------------------------------------------------------------------------------
void	ABVARunner::setGameScreenSize( ABint  inWidth, ABint  inHeght, ABbool  inMinimamCancel ){

	//	݂̃EBhETCYƔrAEBhETCYΑ傫B
	ABint			maxLength;
	ABint			sw, sh;
	ABuint32		w, h;
	GUIView*		view = GameGetView();
	view->getCurrentViewSize( &w, &h );
	if( inWidth < inHeght){
		maxLength = inHeght;
	}
	else{
		maxLength = inWidth;
	}
	
	ABscreenMode	modeCount;
	for( modeCount = AB_SCREEN_SIZE_MODE_320_240; modeCount < (AB_SCREEN_SIZE_MODE_COUNT - 1) ; modeCount++){
		AbConfigGetScreenSizeMode( modeCount, &sw, &sh );
		if( maxLength <= sw || maxLength <= sh ){
			//	SɎ܂TCYł̂ŁÃTCYɕύX
			break;
		}
	}
	//	Ă͂܂ȂƂ́AƂ傫ȃTCYƂ
	
	this->m_screenWidth = inWidth;
	this->m_screenHeight = inHeght;

	GGGetRender().setGameScreenSize( inWidth, inHeght );

	if( inMinimamCancel ){

		//	݂̉𑜓x傫Ȃ΂ȂȂƂAύX
		if( ! (w <= sw && h <= sh) ){
			return;
		}
	}

	if( sw != w && sh != h ){
		//	ۂɉ𑜓x؂ւ
		ABGetPreference().playConfig.screenMode = modeCount;
		view->changeScreenSize( sw, sh );
	}
}
void	ABVARunner::getGameScreenSize( ABint*  outWidth, ABint*  outHeght ){
	*outWidth = this->m_screenWidth;
	*outHeght = this->m_screenHeight;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS	ABVASchemeSetupper
//
//////////////////////////////////////////////////////////////////////////////////////////////////

//	Xgw
ABbool	ABVASchemeSetupper::setList( ABuint  inListNumber )
{
	if( ABVA_ITEM_LIST_MAX < inListNumber ){
		return false;
	}
	
	this->m_currentList			= &this->m_ruuner.m_itemList[ inListNumber ];
	this->m_currentItemArray	= &this->m_ruuner.m_itemArray[ inListNumber ];
	this->m_binder				= &this->m_ruuner.m_binderArray[ inListNumber ];
	this->m_renderer			= &this->m_ruuner.m_renderer[ inListNumber ];
	
	return true;
}

//	oC_[ݒ
//		Eǉ ABVAItemBinder ́AsvɂȂ_ŎI delete ܂B
ABbool	ABVASchemeSetupper::bind( ABVAItemBinder*  inBinder, ABVAItemRender*  inRenderer )
{
	*this->m_binder = inBinder;
	*this->m_renderer = inRenderer;
	return true;
}

//	ǉ
ABVAItem*	ABVASchemeSetupper::addItem( ABVAItem*  inItem, ABint  inItemNumber ) throw( std::bad_alloc ) 
{
	ABVAItemListIt		it;
	
	if( this->m_currentList == NULL ){
		throw   std::bad_alloc();
	}
	
	inItem->mItemID = inItemNumber;
	this->m_currentList->push_back( inItem );
	
	if( 0 <= inItemNumber ){
		it = this->m_currentList->end();
		it--;
		if( this->m_currentItemArray->size() <= inItemNumber ){
			this->m_currentItemArray->resize( inItemNumber + 1 );
		}
		(*this->m_currentItemArray)[ inItemNumber ] = it;
	}
	return	inItem;
}
