/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		ABMBScheme.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////


#include	<iostream>
#include	"GlobalConfig.h"
#include	"ABMBScheme.h"

//#include	"GEMacFileInfo.h"

/*//////////////////////////////////////////////////////////////////////////////
*	class
*///////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------
/*
*	Constructor
*/
//-------------------------------------------------------
ABMBScheme::ABMBScheme()
{
}
//-------------------------------------------------------
/*
*	Destructor
*/
//-------------------------------------------------------
ABMBScheme::~ABMBScheme()
{
	this->closeScheme();
}

//-------------------------------------------------------
//	open scheme directory
//-------------------------------------------------------
ABbool	ABMBScheme::openSchemeLibrary( ABint8*  inDirectory )
{
	GameFileInfoList*		fileArray	= NULL;
	GameFileInfoListIt		it;

	GameFileInfo*			fileInfo	= NULL;
	ABbool					result		= false;

	this->m_scemeNumber = 0;
	this->m_schemeDirectroy.make( (ABuint8*)inDirectory );

	//	collect folder
	result = this->m_scemeList.collectFilInfo( inDirectory, true );
	if( !result ){
		return result;
	}
	
	this->m_trueIsOpened = false;
	return	result;
}

//-------------------------------------------------------
//	get sceme count
//-------------------------------------------------------
ABint		ABMBScheme::getCount( void )
{
	return	this->m_scemeList.getCount();
}

//-------------------------------------------------------
//	get number from folder name
//-------------------------------------------------------
ABint	ABMBScheme::getSchemeNumber( ABint8*  inFolderName )
{
	//	t@CXgvtH_[
	GameFileInfoListIt	it;
	ABint	counter = 0;
	ABint8	fileName[ AB_BMS_CHAR_LENGTH_FILEPATH ];
	for( it = this->m_scemeList.getArray()->begin(); it != this->m_scemeList.getArray()->end(); it++){
		GameFileInfo*		file = &(*it);
		file->copyName( fileName, sizeof(fileName) );
		if( 0 == strcmp( inFolderName, fileName ) ){
			//	v
			return	counter;
		}
		++counter;
	}
	//	
	ABReportError( "not found scheme folder." );
	return -1;
}

//-------------------------------------------------------
//	open sceme data
//-------------------------------------------------------
ABbool	ABMBScheme::openScheme( ABint  inNumber )
{
	ABbool			result = false;
	GameFileInfo*	fileInfo;
	ABMBSchemeSet*	scemeSetPtr = NULL;

	if( this->m_scemeList.getCount() < inNumber){
		return	false;
	}

	//	wԍXL[I
	fileInfo = this->m_scemeList.getInfo( inNumber );
	this->m_scemeFolder.copy( *fileInfo );
	
	//	init
	this->m_schemeArray.clear();
	this->m_schemeArray.reserve( AB_SCHEME_SET_COUNT );

	//	ԍscan scheme folder
	result = this->scanScemeFolder( fileInfo );
	ABTestReturnReportError( ! result, "scanScemeFolder." );

	//	read scheme
	result = this->m_parser.readScheme( &this->m_scemeSetFile, &this->m_schemeArray );
	ABTestReturnReportError( ! result, "this->m_parser.readScheme." );
	
	// 	get Resource
	for( ABint i = 0 ; i < this->m_schemeArray.size(); i++){
		scemeSetPtr = &this->m_schemeArray[i];
		result = this->openResource( fileInfo, scemeSetPtr );
		if( !result ){
			//	close
			this->closeResource();
			ABReportError( "openResource." );
			return	false;
		}
	}
	
	this->m_scemeNumber = inNumber;
	this->m_trueIsOpened = true;
	return	true;
}

//-------------------------------------------------------
//	close sceme data
//-------------------------------------------------------
void	ABMBScheme::closeScheme( void )
{
	if( this->m_trueIsOpened ){
		this->closeResource();
		this->m_trueIsOpened = false;
	}
}

//-------------------------------------------------------
//	get sceme data
//-------------------------------------------------------
ABMBSchemeSet*	ABMBScheme::getScheme( ABint  inSetNumber )
{
	if( this->m_schemeArray.size() < inSetNumber ){
		return	NULL;	
	}
	else{
		return	&( this->m_schemeArray[inSetNumber] );
	}
}

//-------------------------------------------------------
//	get sceme count
//-------------------------------------------------------
ABint	ABMBScheme::getSchemeCount( void )
{
	if( this->m_trueIsOpened ){
		return	this->m_schemeArray.size();
	}
	return	0;
}

//-------------------------------------------------------
//	scan scheme foleder
//-------------------------------------------------------
ABbool	ABMBScheme::scanScemeFolder( GameFileInfo*  inFolder )
{
	GameFileInfoList*		fileArray = NULL;
	GameFileInfoListIt		it;

	const int		len = AB_BMS_CHAR_LENGTH_FILEPATH;

	ABint8			fileName[ len + 1];
	ABint8*			fileNamePtr;
	ABint8			fullpath[AB_BMS_CHAR_LENGTH_FILEPATH + 1];
	ABbool			result;
	int				extCharLen = strlen( AB_SCEME_SET_EXT );
	GameFileList	schemeData;
		
	//	get path
	result = inFolder->copyPath( (ABuint8*)fullpath, AB_BMS_CHAR_LENGTH_FILEPATH );
	if( !result ){ return false; }

	//	collect folder
	result = schemeData.collectFilInfo( fullpath, false );
	if( !result ){ return false; }

	//	search ".mbscm" text
	fileArray = schemeData.getArray();
	for( it = fileArray->begin() ; it != fileArray->end() ; it++ ){
/*
		ABuint8	c[1024];
		FSRef	fsref =it->getFSRef();
		GEMacFileTool::getPathFromFSRef( &fsref, c, 1024 );
		printf( "%s\n", c );
*/
		memset( fileName, 0, sizeof(fileName) );
		result = it->copyPath( (ABuint8*)fileName, len );
		
		if( extCharLen <= strlen(fileName)  ){
			fileNamePtr = strrchr( fileName, 0 );
			fileNamePtr -= extCharLen;
			
			if( 0 == strcasecmp( fileNamePtr, AB_SCEME_SET_EXT ) ){
				//	hit
				break;
			}
		}
	};
	
	if( it == fileArray->end() ){
		return false;
	}
	
	this->m_scemeSetFile.copy( *it );
	return true;
}
//-------------------------------------------------------
//	open pics data / get on off screen buffer
//-------------------------------------------------------
ABbool	ABMBScheme::openResource( GameFileInfo*  inScemeFolder, ABMBSchemeSet*  inSet )
{
	ABbool				result;
	int					count = 0;
	int					mode = 0;
	ResPictKey			resNum;
	ABMBSchemeResource*	schemeRes;

	ABint8				fullPath[AB_BMS_CHAR_LENGTH_FILEPATH];
	ABint8				sceneFolderPath[AB_BMS_CHAR_LENGTH_FILEPATH];
	ABint8				resFolderPath[AB_BMS_CHAR_LENGTH_FILEPATH];

	ABint8*  			resourceFolder = inSet->ResDirName;
	
	//	resouace folder
	memset( resFolderPath, 0, sizeof(resFolderPath) );
	memset( sceneFolderPath, 0, sizeof(sceneFolderPath) );

	result = inScemeFolder->copyPath( (ABuint8*)sceneFolderPath, AB_BMS_CHAR_LENGTH_FILEPATH  );
	if( !result ){ return false; }

	this->makeFullpath( resFolderPath, sceneFolderPath, resourceFolder, strlen(sceneFolderPath) );

	//	Resource\̂T
	for( mode = SCMode_start ; mode < SCMode_end ; mode++ ){
	
		for( resNum = ResPict_start ; resNum < ResPict_end ; resNum++ ){

			//	ResourceQ
			schemeRes = inSet->getResource( mode, resNum );
			if( schemeRes != NULL ){
			
				if( ! schemeRes->isSet ){
					//	tpX쐬
					memset( fullPath, 0, AB_BMS_CHAR_LENGTH_FILEPATH );
					this->makeFullpath( fullPath, resFolderPath, schemeRes->name, strlen(resFolderPath) );
					if( 0 != strlen( fullPath ) ){
	
						//	t@Ci[
						schemeRes->file.make( (ABuint8*)fullPath );
						
						//	get
						ABDebug( std::cout << "Open Res::" << "Mode:" << mode << ",Num:" << resNum << ",Path:" << fullPath << std::endl );

						schemeRes->isSet = true;
						++count;

					}
				}

			}
		}
	}

	ABTestReturnReportError( count == 0, "empty resource files." );
	return true;
}
//-------------------------------------------------------
//	close pics dara
//-------------------------------------------------------
void	ABMBScheme::closeResource( void )
{
	//	摜ǗVAɈڍs
/*
	int					mode = 0;
	int					resNum = 0;
	ABMBSchemeResource*	schemeRes;

	for( mode = SCMode_start ; mode < SCMode_end ; mode++ ){
		for( resNum = ResPict_start ; resNum < ResPict_end ; resNum++ ){
			schemeRes = this->m_schemeArray[0].getResource( mode, resNum );
			if( schemeRes != NULL ){
				if( schemeRes->isSet ){
					//	Ȃ
				}
			}
		}
	}
*/
}

//-------------------------------------------------------
//	make fullpath
//-------------------------------------------------------
void	ABMBScheme::makeFullpath( ABint8*  outFullPath , ABint8*  inFolderName, ABint8*  inFileName ,ABint  inFolderNameSize )
{
	//	make path	/*::TODO::I[o[l鎖*/
	if( 0 == strlen(inFileName) ){
		outFullPath = NULL;
		return;
	}
	if( outFullPath[ inFolderNameSize - 1 ] != '/' ){
		sprintf( outFullPath , "%s/%s", inFolderName, inFileName );
	}
	else{
		sprintf( outFullPath , "%s%s", inFolderName, inFileName );
	}
}

