/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		BMSFolder.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	"BMSFolder.h"
#include	"BMSResource.h"
#include	"BMSSong.h"
#include	"BMSParser.h"
#include	"BMSProgressReporter.h"

//-------------------------------------------------------
//	Constructor
//-------------------------------------------------------
BMSFolder::BMSFolder()
:
	m_resource( NULL ),
	m_songData( NULL ),
	m_song( NULL )
{
}
//-------------------------------------------------------
//	Destructor
//-------------------------------------------------------
BMSFolder::~BMSFolder()
{
	this->closeFolder();
}

////////////////////////////////////////////////////////////////////////////////
//	BMSt@C
////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------
//	BMStH_J
//-------------------------------------------------------
ABbool		BMSFolder::openFolder( ABint8*  inFolderPath )
{
	this->closeFolder();

	try{
		this->m_resource	= new BMSResource;
	}
	catch( std::bad_alloc ){
		ABReportError( "BMSResource bad alloc." );
		return false;
	};
	
	return	this->m_resource->collectSong( inFolderPath );
}

//-------------------------------------------------------
//	BMStH_\O擾
//-------------------------------------------------------
ABuint		BMSFolder::countSong( void )
{
	if( this->m_resource == NULL ){
		ABReportError( "this->m_resource empty." );
		return false;
	}
	return	this->m_resource->countSong();
}

//-------------------------------------------------------
//	BMS\O擾
//-------------------------------------------------------
ABbool		BMSFolder::getInformation( const ABuint  inSongNumber, BMSTPSongInformation*  outSongInformation )
{
	if( this->m_resource == NULL ){
		ABReportError( "this->m_resource empty." );
		return false;
	}
	if( this->m_resource->countSong() <= inSongNumber ){
		ABReportError( "faital argument inSongNumber." );
		return false;
	}

	BMSParser		parser;
	BMSTPSongData	songTextData;
	if( ! parser.loadBMSFile( this->m_resource, BMSResource::openStageTitleBMP, 
								NULL, BMS_TP_LOAD_HEADER_INFO_DATA, this->m_resource->getSongFile( inSongNumber ), &songTextData ) ){
		ABReportError( "BMSParser loadBMSFile." );
		return false;
	}

	*outSongInformation = songTextData.information;
	return	true;
}

//-------------------------------------------------------
//	BMSt@CJ
//-------------------------------------------------------
BMSSong*		BMSFolder::openSong( BMSProgressReporter*  inReporter, const ABuint  inSongNumber, const ABbool  inDisableBMP, const  ABbool  inIntroOnly )
{
	if( this->m_resource == NULL ){
		ABReportError( "this->m_resource empty." );
		return NULL;
	}
	if( this->m_resource->countSong() <= inSongNumber ){
		ABReportError( "faital argument inSongNumber." );
		return NULL;
	}
	
	this->closeSong();

	try{
		this->m_songData= new BMSTPSongData;
		this->m_song	= new BMSSong( *this->m_songData );
	}
	catch( std::bad_alloc ){
		ABReportError( "BMSSong bad alloc." );
		return NULL;
	};

	inReporter->begin( 2, this->m_resource );	//	eLXgAt@Cǂݎ̂QtF[Y

	//	Parserŉ
	BMSParser	parser;
	if( ! parser.loadBMSFile( this->m_resource, BMSResource::openStageTitleBMP, inReporter, BMS_TP_LOAD_ALL_DATA, this->m_resource->getSongFile( inSongNumber ), this->m_songData ) ){
		ABReportError( "BMSParser loadBMSFile." );
		return NULL;
	}


	//	ԃf[^
	ABbool	ret = this->m_song->analyze();
	if( !ret ){
		ABReportError( "Song analyze." );
		return NULL;
	}
	
	//	ResourceJ
	BMSTPSongData*			songData = this->m_song->getSongData();

//	songData->midiFileData = this->m_resource->openWAV( songData->fileNameMidi )

	ABint	errorCountBMP = 0;
	ABint	errorCountWAV = 0;
	ABint	counter = 0;
	ABint	fileCount = songData->materialBMPArray.size() + songData->materialWAVArray.size();
	ABint	position = 0;
	inReporter->countPhaze();
	inReporter->onReport( fileCount , position );

	BMSMaterialBMPArrayIt	bit;
	bit = songData->materialBMPArray.begin();
	//	songData->materialBMPArraýAIuWFNgzlŒł悤zɂȂĂB
	//	āAz0łŏ̉摜́A~X摜łB
	if( bit->filename[0] != 0 ){
		songData->information.missBMPData = this->m_resource->openBMP( bit->filename );
		printf( "Open MISS BMP file [%4d][%s][data:%d]\n", counter, bit->filename, songData->information.missBMPData );
		if( bit->bmpData == 0 ){
			++errorCountBMP;
		}
		inReporter->onReport( fileCount , position );	++position;
	}
	++counter;

	//	c摜	
	bit++;
	for( ; bit != songData->materialBMPArray.end(); bit++ ){
		if( bit->filename[0] != 0 ){
			bit->bmpData = this->m_resource->openBMP( bit->filename );
			printf( "Open BMP file [%4d][%s][data:%d]\n", counter, bit->filename, bit->bmpData );
			if( bit->bmpData == 0 ){
				++errorCountBMP;
			}
		}
		inReporter->onReport( fileCount , position );	++position;
		++counter;
	}

	BMSMaterialWAVArrayIt	wit;
	counter = 0;
	for( wit = songData->materialWAVArray.begin(); wit != songData->materialWAVArray.end(); wit++ ){
		if( wit->filename[0] != 0 ){
			wit->wavData = this->m_resource->openWAV( wit->filename );
			ABDebug( printf( "Open WAV file [%4d][%s][data:%d]\n", counter, wit->filename, wit->wavData ); );
			if( wit->wavData == 0 ){
				++errorCountWAV;
			}
		}
		inReporter->onReport( fileCount , position );	++position;
		++counter;
	}

	if( errorCountWAV != 0 || errorCountWAV != 0 ){
		ABReportError( "any file oepn error." );
	}
	
	//	ocashing
	//this->m_resource->cashingWAV();
	
	inReporter->end();
	return this->m_song;
}

//-------------------------------------------------------
//	BMSt@C
//-------------------------------------------------------
void		BMSFolder::closeSong( void )
{
	if( this->m_resource == NULL ){
		return;
	}
	if( this->m_song == NULL ){
		return;
	}

	BMSTPSongData*			songData = this->m_song->getSongData();
	BMSMaterialBMPArrayIt	bit;
	for( bit = songData->materialBMPArray.begin(); bit != songData->materialBMPArray.end(); bit++ ){
		this->m_resource->closeBMP( bit->bmpData );
	}
	BMSMaterialWAVArrayIt	wit;
	for( wit = songData->materialWAVArray.begin(); wit != songData->materialWAVArray.end(); wit++ ){
		this->m_resource->closeWAV( wit->wavData );
	}
	if( 0 < songData->information.midiFileData ){
		this->m_resource->closeWAV( songData->information.midiFileData );
	}
	if( 0 < songData->information.stageBMPData ){
		this->m_resource->closeBMP( songData->information.stageBMPData );
	}
	if( 0 < songData->information.missBMPData ){
		this->m_resource->closeBMP( songData->information.missBMPData );
	}

	this->m_song->clear();

	delete	this->m_song;
	this->m_song = NULL;
	delete	this->m_songData;
	this->m_songData = NULL;

	ABDebug( ABReport( "closeSong." ); );
}

//-------------------------------------------------------
//	BMStH_
//-------------------------------------------------------
void		BMSFolder::closeFolder( void )
{
	this->closeSong();

	if( this->m_resource ){
		delete	this->m_resource;
		this->m_resource = NULL;
	}
}
