/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		BMSFolder.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	"BMSFolder.h"
#include	"BMSResource.h"
#include	"BMSSong.h"
#include	"BMSParser.h"

//-------------------------------------------------------
//	Constructor
//-------------------------------------------------------
BMSFolder::BMSFolder()
{
}
//-------------------------------------------------------
//	Destructor
//-------------------------------------------------------
BMSFolder::~BMSFolder()
{
}

////////////////////////////////////////////////////////////////////////////////
//	BMSt@C
////////////////////////////////////////////////////////////////////////////////

//-------------------------------------------------------
//	BMStH_J
//-------------------------------------------------------
ABbool		BMSFolder::openFolder( ABint8*  inFolderPath )
{
	this->closeFolder();

	try{
		this->m_resource	= new BMSResource;
	}
	catch( std::bad_alloc ){
		ABReportError( "BMSResource bad alloc." );
		return false;
	};
	
	return	this->m_resource->collectSong( inFolderPath );
}

//-------------------------------------------------------
//	BMStH_\O擾
//-------------------------------------------------------
ABuint		BMSFolder::countSong( void )
{
	if( this->m_resource == NULL ){
		ABReportError( "this->m_resource empty." );
		return false;
	}
	return	this->m_resource->countSong();
}

//-------------------------------------------------------
//	BMStH_\OI
//-------------------------------------------------------
ABbool		BMSFolder::selectSong( const ABuint  inSongNumber )
{
	if( this->m_resource == NULL ){
		ABReportError( "this->m_resource empty." );
		return false;
	}

	this->closeSong();
	
	try{
		this->m_song	= new BMSSong;
	}
	catch( std::bad_alloc ){
		ABReportError( "BMSSong bad alloc." );
		return false;
	};

	//	Parserŉ
	BMSParser	parser;
	if( ! parser.loadBMSFile( BMS_TP_LOAD_ALL_DATA, this->m_resource->getSongFile( inSongNumber ), this->m_song ) ){
		ABReportError( "BMSParser loadBMSFile." );
		return false;
	}
	
	return	true;
}

//-------------------------------------------------------
//	BMS\O擾
//-------------------------------------------------------
ABbool		BMSFolder::getInformation( BMSTPSongInformationPtr*  outSongInformation )
{
	if( this->m_resource == NULL ){
		ABReportError( "this->m_resource empty." );
		return false;
	}
	if( this->m_song == NULL ){
		ABReportError( "this->m_song empty." );
		return false;
	}

	//	^Cg摜J
	BMSTPSongData*			songData = this->m_song->getSongData();
	if( 0 < songData->information.stageBMPData ){
		this->m_resource->closeBMP( songData->information.stageBMPData );
	}
	songData->information.stageBMPData = this->m_resource->openBMP( songData->information.fileNameStageFile );

	*outSongInformation = &( songData->information );
	return	true;
}

//-------------------------------------------------------
//	BMSt@CJ
//-------------------------------------------------------
ABbool		BMSFolder::openSong( const ABbool  inDisableBMP, const  ABbool  inIntroOnly )
{
	if( this->m_resource == NULL ){
		ABReportError( "this->m_resource empty." );
		return false;
	}
	if( this->m_song == NULL ){
		ABReportError( "this->m_song empty." );
		return false;
	}
	
	this->closeSong();

	//	ԃf[^
	ABbool	ret = this->m_song->analyze();
	ABTestReturnReportError( !ret, "Song analyze." );
	
	//	ResourceJ
	BMSTPSongData*			songData = this->m_song->getSongData();

//	songData->midiFileData = this->m_resource->openWAV( songData->fileNameMidi )

	ABint	errorCountBMP = 0;
	ABint	errorCountWAV = 0;

	BMSMaterialBMPArrayIt	bit;
	for( bit = songData->materialBMPArray.begin(); bit != songData->materialBMPArray.end(); bit++ ){
		bit->bmpData = this->m_resource->openBMP( bit->filename );
		if( bit->bmpData == 0 ){
			++errorCountBMP;
		}
	}

	BMSMaterialWAVArrayIt	wit;
	for( wit = songData->materialWAVArray.begin(); wit != songData->materialWAVArray.end(); wit++ ){
		wit->wavData = this->m_resource->openWAV( wit->filename );
		if( wit->wavData == 0 ){
			++errorCountWAV;
		}
	}

	if( errorCountWAV != 0 || errorCountWAV != 0 ){
		ABReportError( "any file oepn error." );
	}
	
	return true;
}

//-------------------------------------------------------
//	BMSt@C
//-------------------------------------------------------
void		BMSFolder::closeSong( void )
{
	if( this->m_resource == NULL ){
		return;
	}
	if( this->m_song == NULL ){
		return;
	}

	BMSTPSongData*			songData = this->m_song->getSongData();
	BMSMaterialBMPArrayIt	bit;
	for( bit = songData->materialBMPArray.begin(); bit != songData->materialBMPArray.end(); bit++ ){
		this->m_resource->closeBMP( bit->bmpData );
	}
	BMSMaterialWAVArrayIt	wit;
	for( wit = songData->materialWAVArray.begin(); wit != songData->materialWAVArray.end(); wit++ ){
		this->m_resource->closeWAV( wit->wavData );
	}
	if( 0 < songData->information.midiFileData ){
		this->m_resource->closeWAV( songData->information.midiFileData );
	}
	if( 0 < songData->information.stageBMPData ){
		this->m_resource->closeBMP( songData->information.stageBMPData );
	}
	this->m_song->clear();
	songData->clean();
}

//-------------------------------------------------------
//	BMStH_
//-------------------------------------------------------
void		BMSFolder::closeFolder( void )
{
	if( this->m_resource == NULL ){
		return;
	}
	this->m_resource->close();
}
