/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		BMSParser.h

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#ifndef	BMSParser_h
#define	BMSParser_h

#include	"GlobalConfig.h"
#include	"GameEngine.h"
#include	"GameFile.h"

#include	"BMSConfig.h"
#include	"BMSParserConfig.h"
#include	"GameTextParser.h"

#include	<vector>
#include	<algorithm>

class	BMSSong;
class	BMSProgressReporter;
class	BMSResource;

//	eLXgParserǂݍݎAwb_[Ă񂾌ɌĂԊ֐B
//	̃^C~OŃXe[Wt@CJ܂
typedef	ABbool	(*BMSParserHeaderReafAfterProc)( BMSResource*  inResource, BMSTPSongData*  ioTPSongData );

/*//////////////////////////////////////////////////////////////////////////////
*	class AB BMS Music Analyzer
*///////////////////////////////////////////////////////////////////////////////

//	BMSf[^͂dg݂JX^}CY₷`Œ񋟂B
//
//	ECxg̎ނ΁AoĂǂɂǂ̂AƂANV`łAƂlB
//	@ȉ͂̃ANVi񋟂B
//	ExANVpӂĂ΁Aǂݍݎswitch肷KvȂȂB
//
//--------------------------------------------------------------
//	class
//--------------------------------------------------------------
class	BMSTPArchitectureBase
{
public:
	BMSTPArchitectureBase(){
		this->mEventType = BMS_TP_EVENT_NOTHING;
	};
	virtual ~BMSTPArchitectureBase(){};

	//	Cxg̉͂Ǝ擾܂BŒl̎擾AIuWFNgz̉͂sB
	//	l͕̂܂ܓn̂ŁAKvɉď邱ƁB
	virtual bool	analyze( BMSTPVibrantData&  outEventData, ABint  inBarID, BMSTPEventType  inEventType, char*  inValue ){
		return true;
	};

	//	rp
	static bool	compare( BMSTPArchitectureBase*  l, BMSTPArchitectureBase*  r ){
		return	l->mEventType < r->mEventType;
	}
	static bool	equal( BMSTPArchitectureBase*  l, BMSTPArchitectureBase*  r ){
		return	l->mEventType == r->mEventType;
	}
	
	//	
	void	setData( 
		BMSTPEventType		inEventType
	)
	{
		this->mEventType = inEventType;
	};
	BMSTPEventType		mEventType;
};

/*//////////////////////////////////////////////////////////////////////////////
*	class AB BMS Music Analyzer
*///////////////////////////////////////////////////////////////////////////////
class	BMSTPMusicArchitecture
{
public:
	enum{
		M_MAX_EVENT_TYPE_COUNT,	
	};
public:
	BMSTPMusicArchitecture(){
		this->mArchitectureArrayTable.resize( BMS_TP_EVENT_MAX, -1 );
	};
	virtual	~BMSTPMusicArchitecture(){};

	//	hNXConstructorŌĂԂƁB
	virtual	void init( void ){
		this->setupDataFormat();
		sort( this->mArchitectureArray.begin(), this->mArchitectureArray.end(), BMSTPArchitectureBase::compare );
	}
	
	//	`
	//	̒ŕKvȒ`Lq܂B
	virtual	void setupDataFormat( void ) = 0;

	//	setupDataFormatŎgpݒ
	void	setData( 
		BMSTPArchitectureBase*  	inArchitecture,
		BMSTPEventType				inEventType
	)
	{
		inArchitecture->setData( inEventType );
		this->addArchitecture( inArchitecture );
	};

public:	
	bool	analyze( BMSTPVibrantData&  outEventData, ABint  inBarID, BMSTPEventType  inEventType, char*  inValue ){
		ABint	index = this->mArchitectureArrayTable[ inEventType ];
		if( index < 0 ){
			ABReportError("-index is under 0.");
			return false;
		}
		return this->mArchitectureArray[index]->analyze( outEventData, inBarID, inEventType, inValue );
	}
	
	typedef	std::vector<BMSTPArchitectureBase*>		MBMSArchitectureArray;
	typedef	std::vector<ABint>						MBMSArchitectureArrayTable;	//	tQƃe[uBQƎԂ萔ɂB
	MBMSArchitectureArray		mArchitectureArray;
	MBMSArchitectureArrayTable	mArchitectureArrayTable;
private:
	void	addArchitecture( BMSTPArchitectureBase*  inArchitecture ){
		ABint	size = this->mArchitectureArray.size();
		this->mArchitectureArray.push_back( inArchitecture );
		this->mArchitectureArrayTable[ inArchitecture->mEventType ] = size;
		
	}
};


////////////////////////////////////////////////////////////////////////////////
//	class
////////////////////////////////////////////////////////////////////////////////

enum{
	BMS_TP_LOAD_ALL_DATA,			//	default
	BMS_TP_LOAD_HEADER_INFO_DATA,	//	header information data only	^CgȂǂ̏̂ݓǂݍ
};
typedef	ABtype	BMSLoadMode;


class	BMSParser
{
public:
	BMSParser();
	virtual	~BMSParser();

	//	BMSt@Cǂݍ
	bool	loadBMSFile( 
				BMSResource*	inResource, 
				BMSParserHeaderReafAfterProc	inProc, 
				BMSProgressReporter*	inReporter, 
				BMSLoadMode		inLoadMode, 
				GameFileInfo*	inFileInfoPtr, 
				BMSTPSongData*	ioTPSongData );


private:
	typedef	GameTextParser::MVecTextArray		MVecTextArray;	//	obt@̍s̐擪obt@i[i`̂݁j

private:
	GameTextParser				m_parserTool;
	BMSTPMusicArchitecture* 	m_musicArchitecture;

	//	BMSCxgʎq`
	void	setupEventTable( void );

	//	eLXg -> wb_[f[^
	bool	readBMSheaderData( BMSLoadMode  inLoadMode, BMSTPSongData*  refMusicData, MVecTextArray*  inArray );

	//	eLXg -> Cxgf[^`
	bool	readBMSeventData( BMSLoadMode  inLoadMode, BMSProgressReporter*  inReporter, BMSTPSongData*  refMusicData, MVecTextArray*  inArray );

public:
	//	擾
	static void	getBufferValue( char*  inBuffer, char*  outValue, int  inLength );
	//	POi擾(long^)
	static void	getBufferValue( char*  inBuffer, long*  outValue );
	//	POi擾(double^)
	static void	getBufferValue( char*  inBuffer, double*  outValue );
	//	PUi擾(unsigned long^)
	static void	getBufferValue( char*  inBuffer, unsigned long*  outValue );
	//	io[ɕϊ
	static int	convCharToNum( char*  inBuffer, int  inLength );
};

#endif	//	BMSParser_h
