/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		GGRender.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	<stdio.h>
#include	<algorithm>
#include	"GGRender.h"
#include	"GGSprite.h"

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	Primitive
//
//////////////////////////////////////////////////////////////////////////////////////////////////
bool	GGPrimitive::zComp( const GGPrimitive&  a, const GGPrimitive&  b )
{
	return  a.sprite->m_locate.z < b.sprite->m_locate.z ;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS GGRender
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
GGRender::GGRender( void )
:
	m_viewPortWidth( 0 ),
	m_viewPortHeight( 0 ),
	m_vertexArray( NULL ),
	m_quadCount( 0 ),
	m_screenWidth( 640 ),
	m_screenHeight( 480 ),
	m_setupFlag( false )
{
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
GGRender::~GGRender( void )
{
}

//-------------------------------------------------------------------------------------------------
//	initialize
//-------------------------------------------------------------------------------------------------
ABbool	GGRender::initialize( const ABsize  inPrimitiveCount )
{
	ABbool	ret = false ;
	do{
		//	_z쐬
		this->m_vertexArray = (GLfloat*)calloc( sizeof(GLfloat), M_VA_ROW_STRIDE * M_QUAD_VERTEX_COUNT * inPrimitiveCount );
		if( this->m_vertexArray == NULL ){
			break;
		}
		
		this->m_quadCount = inPrimitiveCount;
		this->m_fadeLevel = 1.0f;

		ret = true;
	}while( false );

	return	ret;
}

//-------------------------------------------------------------------------------------------------
//	finalize
//-------------------------------------------------------------------------------------------------
void	GGRender::finalize( void )
{
	if( this->m_vertexArray == NULL ){
		free( this->m_vertexArray );
		this->m_vertexArray = NULL;
	}
}

//-------------------------------------------------------------------------------------------------
//	render setup
//-------------------------------------------------------------------------------------------------
void	GGRender::renderSetup( void )
{
	//	_zANZX̊g
	glEnableClientState( GL_VERTEX_ARRAY_RANGE_APPLE );
	glEnableClientState( GL_ELEMENT_ARRAY_APPLE );
	glVertexArrayParameteriAPPLE( GL_VERTEX_ARRAY_STORAGE_HINT_APPLE, GL_STORAGE_SHARED_APPLE );
	glEnableClientState( GL_VERTEX_ARRAY );
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );

	//	_zݒ
	GLsizei		size = M_VA_ROW_STRIDE * M_QUAD_VERTEX_COUNT * sizeof(GLfloat) * this->m_quadCount;
	glVertexArrayRangeAPPLE( size,  this->m_vertexArray );
	glFlushVertexArrayRangeAPPLE( size,  this->m_vertexArray );
	this->m_setupFlag = true;
}

//-------------------------------------------------------------------------------------------------
//	render cleanup
//-------------------------------------------------------------------------------------------------
void	GGRender::renderCleanup( void )
{
	if( this->m_setupFlag ){
		//	_zANZX̏IiĂ񂾌Athis->m_vertexArray̍폜sƁj
		glVertexArrayRangeAPPLE( 0 ,  this->m_vertexArray );

		glDisableClientState( GL_ELEMENT_ARRAY_APPLE );
		glDisableClientState( GL_VERTEX_ARRAY_RANGE_APPLE );
		
		glDisableClientState( GL_VERTEX_ARRAY );
		glDisableClientState( GL_TEXTURE_COORD_ARRAY );
		this->m_setupFlag = false;
	}
}

//-------------------------------------------------------------------------------------------------
//	Render
//-------------------------------------------------------------------------------------------------
void	GGRender::render( void )
{
	glEnable(GL_BLEND);

	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	this->draw();

	glDisable(GL_BLEND);

	glViewport( 0, 0, this->m_viewPortWidth, this->m_viewPortHeight );
}

//-------------------------------------------------------------------------------------------------
//	clear
//-------------------------------------------------------------------------------------------------
void	GGRender::clear( void )
{
	glEnable(GL_BLEND);
	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDisable(GL_BLEND);
	glViewport( 0, 0, this->m_viewPortWidth, this->m_viewPortHeight );
}

//-------------------------------------------------------------------------------------------------
//	reshape
//-------------------------------------------------------------------------------------------------
void	GGRender::reshape( const ABint16  inWidth, const ABint16  inHeight )
{
	this->m_viewPortWidth = inWidth;
	this->m_viewPortHeight = inHeight;

	glViewport( 0, 0, inWidth, inHeight );
}
	

//-------------------------------------------------------------------------------------------------
//	O[v쐬
//-------------------------------------------------------------------------------------------------
ABbool	GGRender::addGroup( GGPrimitiveGroupListIt*  outIt )
{
	ABbool	ret = false ;
	do{
		GGPrimitiveGroup		group;
		GGPrimitiveGroupListIt	it;

		this->m_primitiveGroupList.push_back( group );
		it = this->m_primitiveGroupList.end();
		it--;
		*outIt = it;
		
		GGPrimitiveList*	plist = &it->m_primitiveList;

		ret = true;
	}while( false );

	return	ret;
}

//-------------------------------------------------------------------------------------------------
//	O[vj
//-------------------------------------------------------------------------------------------------
void	GGRender::removeGroup( const GGPrimitiveGroupListIt&  inGroupIt )
{
	this->m_primitiveGroupList.erase( inGroupIt );
}

//-------------------------------------------------------------------------------------------------
//	O[vPRIMITIVE
//		EsƓZWŃ\[g܂Bi኱Ԃ|܂j
//-------------------------------------------------------------------------------------------------
ABbool	GGRender::addPrimitive(
	GGPrimitiveGroupListIt&	inGroupIt, 
	GGSprite*			inSprite,
	const ABbool		inRequestZSort,
	GGPrimitiveListIt*	ioIt
){
	ABbool	ret = false ;
	do{
	
		GGPrimitive			primitive;
		GGPrimitiveListIt	it;
		GGPrimitiveList*	primitiveList = &inGroupIt->m_primitiveList;
		ABbool				addFlag = false;

		inSprite->setNumber( this->m_uniqueNumber.getNewNumber() );
		if( this->m_quadCount < inSprite->getNumber() ){
			break;
		}

		//	PRIMITIVEݒ		
		memset( &primitive, 0, sizeof(primitive) );
		primitive.sprite = inSprite;

		//	z sort
		if( inRequestZSort ){
		
			//	ZWŌ
			it = upper_bound( primitiveList->begin(), 
								primitiveList->end(), 
								primitive, 
								GGPrimitive::zComp );
			if( it != primitiveList->end() ){
				//	̈ʒuɑ}
				it = primitiveList->insert( it, primitive );
			}
			else{
				addFlag = true;
			}
		}
		else{
			addFlag = true;
		}
		
		if( addFlag ){
			primitiveList->push_back( primitive );
			it = primitiveList->end();
			it--;
		}
		
		this->setPrimitive( it, inSprite );
		*ioIt = it;

		ret = true;
	}while( false );

	return	ret;
}

//-------------------------------------------------------------------------------------------------
//	O[vPRIMITIVE
//		EsƓZWŃ\[g܂Bi኱Ԃ|܂j
//-------------------------------------------------------------------------------------------------
ABbool	GGRender::setPrimitive(
	GGPrimitiveListIt&	inIt, 
	GGSprite*			inSprite
){
	ABbool	ret = false ;
	do{
		ABuint				quardNumber = inSprite->getNumber();
	
		ABuint				offset = quardNumber * M_VA_ROW_STRIDE * M_QUAD_VERTEX_COUNT;
		ABuint				i;
		ABuint				s;
		
		ABfloat32	x1 = inSprite->m_locate.x;
		ABfloat32	y1 = inSprite->m_locate.y;
		ABfloat32	x2 = inSprite->m_locate.x + inSprite->m_width;
		ABfloat32	y2 = inSprite->m_locate.y + inSprite->m_height;

		//	_zXV
		{
			s = offset;
			i = s;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = x1;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = y1;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = inSprite->m_locate.z;
			i = s;
			*( this->m_vertexArray + M_VA_OFFSET_TEXCOORD + i++ ) = inSprite->m_texCoord[0].x;
			*( this->m_vertexArray + M_VA_OFFSET_TEXCOORD + i++ ) = inSprite->m_texCoord[0].y;

			s = offset + M_VA_ROW_STRIDE;
			i = s;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = x1;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = y2;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = inSprite->m_locate.z;
			i = s;
			*( this->m_vertexArray + M_VA_OFFSET_TEXCOORD + i++ ) = inSprite->m_texCoord[0].x;
			*( this->m_vertexArray + M_VA_OFFSET_TEXCOORD + i++ ) = inSprite->m_texCoord[1].y;

			s = offset + M_VA_ROW_STRIDE * 2;
			i = s;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = x2;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = y2;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = inSprite->m_locate.z;
			i = s;
			*( this->m_vertexArray + M_VA_OFFSET_TEXCOORD + i++ ) = inSprite->m_texCoord[1].x;
			*( this->m_vertexArray + M_VA_OFFSET_TEXCOORD + i++ ) = inSprite->m_texCoord[1].y;

			s = offset + M_VA_ROW_STRIDE * 3;
			i = s;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = x2;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = y1;
			*( this->m_vertexArray + M_VA_OFFSET_VERTEX + i++ ) = inSprite->m_locate.z;
			i = s;
			*( this->m_vertexArray + M_VA_OFFSET_TEXCOORD + i++ ) = inSprite->m_texCoord[1].x;
			*( this->m_vertexArray + M_VA_OFFSET_TEXCOORD + i++ ) = inSprite->m_texCoord[0].y;
		}

		//	PRIMITIVEݒ		
		inIt->sprite		= inSprite;

		//	_z̃CfbNXݒB
		inIt->elementIndex[0] = M_QUAD_VERTEX_COUNT * quardNumber + 0;
		inIt->elementIndex[1] = M_QUAD_VERTEX_COUNT * quardNumber + 1;
		inIt->elementIndex[2] = M_QUAD_VERTEX_COUNT * quardNumber + 2;
		inIt->elementIndex[3] = M_QUAD_VERTEX_COUNT * quardNumber + 3;
		
		ret = true;
	}while( false );

	return	ret;
}


//-------------------------------------------------------------------------------------------------
//	O[vPRIMITIVE폜
//-------------------------------------------------------------------------------------------------
void	GGRender::removePrimitive( GGPrimitiveGroupListIt&  inGroup, const GGPrimitiveListIt&  inPrimitive )
{
	this->m_uniqueNumber.releaseNumber( inPrimitive->sprite->getNumber() );
	inGroup->m_primitiveList.erase( inPrimitive );
}

//-------------------------------------------------------------------------------------------------
//	FadeIn, FadeOut
//-------------------------------------------------------------------------------------------------
void	GGRender::fade( ABfloat32  inFadeLebel )
{
	this->m_fadeLevel = inFadeLebel;
}
ABfloat32	GGRender::getFade( void )
{
	return	this->m_fadeLevel;
}

//-------------------------------------------------------------------------------------------------
//	drawing
//-------------------------------------------------------------------------------------------------
void	GGRender::draw( void )
{
	GGPrimitiveGroupListIt		git;
	GGPrimitiveListIt			pit;
	GGPrimitiveList*			primitiveList = NULL;
	GGSprite*					sprite = NULL;
	GLuint						currentTexture = 0;

	ABint16		width, height;
	width	= this->m_screenWidth;		//	zXN[TCYłB`͂̑傫ōsAŏIIɁAr[|[gTCYɐLk܂
	height	= this->m_screenHeight;

	//	񂶂Ē_z̓s
	GLsizei		size = M_VA_ROW_STRIDE * M_QUAD_VERTEX_COUNT * sizeof(GLfloat) * this->m_quadCount;
	glFlushVertexArrayRangeAPPLE( size,  this->m_vertexArray );

	//	GLݒ
	glBlendFunc( GL_SRC_ALPHA ,GL_ONE_MINUS_SRC_ALPHA  );

	glVertexPointer( M_VA_SIZE_VERTEX, GL_FLOAT, M_VA_ROW_STRIDE * sizeof(GLfloat), this->m_vertexArray + M_VA_OFFSET_VERTEX );
	glTexCoordPointer( M_VA_SIZE_TEXCOORD, GL_FLOAT, M_VA_ROW_STRIDE * sizeof(GLfloat), this->m_vertexArray + M_VA_OFFSET_TEXCOORD );

	//	O[v
	for( git = this->m_primitiveGroupList.begin() ; git != this->m_primitiveGroupList.end() ; git++ ){


		primitiveList = &git->m_primitiveList;

		//	_r[
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glScalef( 1.0, -1.0, 1.0 );
		
		glDisable(GL_TEXTURE_2D);

		//	viewPort		640 * 480
		//	game screen		640 * 480

		//	viewPort		800 * 600
		//	game screen		640 * 480

//		GLfloat		widthComp = (GLfloat)this->m_viewPortWidth / (GLfloat)width;
//		GLfloat		heightComp = (GLfloat)this->m_viewPortHeight  / (GLfloat)height;
		
		GLfloat		widthComp = (GLfloat)width / (GLfloat)this->m_viewPortWidth;
		GLfloat		heightComp = (GLfloat)height  / (GLfloat)this->m_viewPortHeight;

		if( git->m_useClip ){
			//	NbsO
			//		ʃTCYς̂ƂAgkB
			//		@Ⴆ640*480ʃC[W800*600ʂɊg債ĕ\邽߁Aȉ̌vZsB

			glViewport( (GLfloat)git->m_clipLeft * widthComp, 
						(GLfloat)this->m_viewPortHeight - (GLfloat)( git->m_clipTop + git->m_clipHeight ) * heightComp, 
						(GLfloat)git->m_clipWidth * widthComp, 
						(GLfloat)git->m_clipHeight * heightComp );

			glOrtho( 0.0f , git->m_clipWidth , 0.0f , git->m_clipHeight , -1024.0 , 1024.0 );
			glScalef( widthComp , heightComp, 1.0f );

			glColor4f(0.0, 0.2, 0.5, 0.2f );
			glBegin(GL_QUADS); 
				glVertex3f( 0.0f , 0.0f , 0.0);
				glVertex3f( 0.0f , 0.0f + git->m_clipHeight , 0.0);
				glVertex3f( 0.0f + git->m_clipWidth, 0.0f + git->m_clipHeight , 0.0);
				glVertex3f( 0.0f + git->m_clipWidth, 0.0f , 0.0);
			glEnd();

		}
		else{
			//	NbsO
			//		ʃTCYς̂ƂAgkB
			//		@Ⴆ640*480ʃC[W800*600ʂɊg債ĕ\邽߁Aȉ̌vZsB
			glViewport( 0, 0, this->m_viewPortWidth, this->m_viewPortHeight );
//			glOrtho( 0.0 , this->m_viewPortWidth , 0.0 , this->m_viewPortHeight , -1024.0 , 1024.0 );
			glOrtho( 0.0 , width , 0.0 , height , -1024.0 , 1024.0 );
//			glScalef( widthComp , heightComp, 1.0f );
		}

		glEnable(GL_TEXTURE_2D);
		glBindTexture( GL_TEXTURE_2D, 0 ) ;

		//	fr[
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
			glLoadIdentity();

			//	PRIMITIVE
			for( pit = primitiveList->begin() ; pit != primitiveList->end() ; pit++ ){
							
				sprite = pit->sprite;
				if( sprite->m_visible ){

					//	eNX`
					if( sprite->m_textureName != 0 ){
						if( sprite->m_textureName != currentTexture ){
							//	KvȂƂoCh
							glBindTexture( GL_TEXTURE_2D, sprite->m_textureName ) ;
							glTexEnvi( GL_TEXTURE_ENV ,GL_TEXTURE_ENV_MODE , GL_MODULATE  ) ;
							currentTexture = sprite->m_textureName;
						}
					}
					else{
						if( sprite->m_textureName != currentTexture ){
							glBindTexture( GL_TEXTURE_2D, 0 ) ;
							currentTexture = sprite->m_textureName;
						}
					}
					
					//	F
					glColor4f( sprite->m_color.r, sprite->m_color.g, sprite->m_color.b, sprite->m_color.a );

					if( sprite->m_doRotate ){
						//	[JWϊ
						glPushMatrix();
			
							//	pƌxቺ܂
							glTranslatef( 	sprite->m_locate.x + sprite->m_rotateCenter.x, 
											sprite->m_locate.y + sprite->m_rotateCenter.y, 
											sprite->m_locate.z + sprite->m_rotateCenter.z );

							glRotatef( sprite->m_rotate.x, sprite->m_rotate.y, sprite->m_rotate.z, 1.0f );

							glTranslatef( 	- sprite->m_locate.x - sprite->m_rotateCenter.x, 
											- sprite->m_locate.y - sprite->m_rotateCenter.y, 
											- sprite->m_locate.z - sprite->m_rotateCenter.z );

							//	_Tu~bg
							//		S_̏CɃTu~bgB
							glElementPointerAPPLE(GL_UNSIGNED_SHORT, pit->elementIndex );
							glDrawElementArrayAPPLE( GL_QUADS , 0, M_QUAD_VERTEX_COUNT );
						
						glPopMatrix();
					}
					else{
						//	]Ȃꍇ͒ڂށB
						//	]̍WvZPRIMITIVEǉɍs悤ɂǂB
						//	iA}gNXẐʓ|Ȃ̂Ŏ蔲j

						//	_Tu~bg
						//		S_̏CɃTu~bgB
						glElementPointerAPPLE(GL_UNSIGNED_SHORT, pit->elementIndex );
						glDrawElementArrayAPPLE( GL_QUADS , 0, M_QUAD_VERTEX_COUNT );
					}

				}
			}

		glPopMatrix();

		glDisable(GL_TEXTURE_2D);

	}
	//	Fader
	if( this->m_fadeLevel < 1.0f ){
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glScalef( 1.0, -1.0, 1.0 );

		glViewport( 0, 0, this->m_viewPortWidth, this->m_viewPortHeight );
		glOrtho( 0.0 , this->m_viewPortWidth , 0.0 , this->m_viewPortHeight , -1024.0 , 1024.0 );

		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
			glLoadIdentity();
			glColor4f( 0.0f, 0.0f, 0.0f, 1.0f - this->m_fadeLevel );
			glBegin( GL_QUADS );
				glVertex3f( 0, 0, -100.0 );
				glVertex3f( this->m_viewPortWidth, 0, -100.0 );
				glVertex3f( this->m_viewPortWidth, this->m_viewPortHeight, -100.0 );
				glVertex3f( 0, this->m_viewPortHeight, -100.0 );
			glEnd();
		glPopMatrix();
	}
}

