/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		GGText.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	"GGText.h"

using namespace code_worker;

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS	GGText
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
GGText::GGText( void )
{
	this->mUseBackColor = false;
	this->mSoftReturn = true;
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
GGText::~GGText( void )
{
	this->cleanup();
}

//-------------------------------------------------------------------------------------------------
//	`
//-------------------------------------------------------------------------------------------------
ABbool	GGText::setup( void )
{
	CWresultCode	result;
	ABbool ret;
	ABint	tw, th;

	ret = this->mSurface.createBitmap( this->mSprite.m_width, this->mSprite.m_height, GE_COLOR_DEPTH_32 );
	if( !ret ){ return false; }
	
	ret = this->mSurface.bindTexture();
	if( !ret ){ return false; }
	
	this->mSurface.getTextureSize( &tw, &th );
//	tw = this->mSprite.m_width;
//	th = this->mSprite.m_height;
	
	this->mSprite.m_textureName = this->mSurface.getTextureName();
	this->mSprite.m_texCoord[0].x = 0.0f;
	this->mSprite.m_texCoord[0].y = 0.0f;
	this->mSprite.m_texCoord[1].x = (ABfloat32)this->mSprite.m_width / (ABfloat32)tw;
	this->mSprite.m_texCoord[1].y = (ABfloat32)this->mSprite.m_height / (ABfloat32)th;
	this->mSprite.update();
	
	
	result = this->mStyle.create( this->mFontName, this->mFontSize );
	if( result != CW_NO_ERROR ){
		return false;
	}

	return true;
}

//-------------------------------------------------------------------------------------------------
//	`I
//-------------------------------------------------------------------------------------------------
void	GGText::cleanup( void )
{
	this->mSprite.cleanup();
	this->mStyle.release();
}

//-------------------------------------------------------------------------------------------------
//	`悷
//-------------------------------------------------------------------------------------------------
ABbool	GGText::draw( GEunicodeTextPtr  inText )
{
	CWresultCode	result;
	CGRect			textBoxRect;
	ABint			w, h;
	GTSLayoutInfo		layout;
	GTStextStyleFace	fase;
	GTSGetDefaultLayoutInfo( &layout );
	
	ABTestReturnReportError( inText == NULL, "inText is NULL." );
	
	switch( this->mFontStyle ){
		case GE_FONT_NOMAL:
			fase = GTS_TEXT_FACE_NORMAL;
			break;
		case GE_FONT_BOLD:
			fase = GTS_TEXT_FACE_BOLD;
			break;
		case GE_FONT_ITALIC:
			fase = GTS_TEXT_FACE_ITALIC;
			break;
	};
	this->mStyle.setFace( fase );
	
	switch( this->mAlignX ){
		case GE_TEXT_ALIGN_X_CENTER:
			layout.alignH = GTS_TEXT_ALIGN_CENTER;
			break;
		case GE_TEXT_ALIGN_RIGHT:
			layout.alignH = GTS_TEXT_ALIGN_END;
			break;
		case GE_TEXT_ALIGN_X_FIT:
			layout.alignH = GTS_TEXT_ALIGN_JUST;
			break;
		default:
		case GE_TEXT_ALIGN_LEFT:
			layout.alignH = GTS_TEXT_ALIGN_START;
			break;
	};
	
	switch( this->mAlignY ){
		case GE_TEXT_ALIGN_Y_CENTER:
			layout.alignV = GTS_TEXT_ALIGN_CENTER;
			break;
		case GE_TEXT_ALIGN_BOTTOM:
			layout.alignV = GTS_TEXT_ALIGN_END;
			break;
		case GE_TEXT_ALIGN_Y_FIT:
			layout.alignV = GTS_TEXT_ALIGN_JUST;
			break;
		default:
		case GE_TEXT_ALIGN_TOP:
			layout.alignV = GTS_TEXT_ALIGN_START;
			break;
	};
	
	layout.useUntiAliasing = this->mUntiAliasing;
	layout.useBreak = this->mSoftReturn;
	
	layout.useBackLayer	= this->mUseBackColor;
	layout.backColorR	= this->mBackColor.r;
	layout.backColorG	= this->mBackColor.g;
	layout.backColorB	= this->mBackColor.b;
	layout.backColorA	= this->mBackColor.a;

	//	Quartzŕ`悷邽߁Aobt@̐mȃTCYKv
	this->mSurface.getTextureSize( &w, &h );
	textBoxRect.size.width	= this->mSprite.m_width;
	textBoxRect.size.height	= this->mSprite.m_height;
	textBoxRect.origin.x = 0;
	textBoxRect.origin.y = 0;

	result = this->mTextBox.setText( &this->mStyle, (CFStringRef)inText );
	if( result != CW_NO_ERROR ){
		return false;
	}

	//	SURFACEɕ`
	this->mSurface.clear();
	CGContextRef	context = this->mSurface.beginDraw();
	result = this->mTextBox.draw( context, &layout, &textBoxRect );
	this->mSurface.flushBuffer();
	this->mSurface.endDraw();
	if( result != CW_NO_ERROR ){
		return false;
	}
	this->mSurface.updateTexture();

	return	true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS	GGTextSprite
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
GGTextSprite::GGTextSprite( void )
{
	mTextLengthMax = 0;	//	ő啶
	memset( &mLocate, 0, sizeof(mLocate) );		//	`Jnʒu
	mWidth = 0;			//	P̃XvCgTCY
	mHeight = 0;		//	P̃XvCgTCY
	mFont = NULL;			//	tHg
	mSurface = NULL;		//	tHgpeNX`
	memset( &mColor, 0, sizeof(mColor) );
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
GGTextSprite::~GGTextSprite( void )
{
	this->cleanup();
}

//-------------------------------------------------------------------------------------------------
//	XvCgZbgAbv
//-------------------------------------------------------------------------------------------------
ABbool	GGTextSprite::setup( GGPrimitiveGroupListIt&  inIt, ABbool  inZorder )
{
	ABbool				ret = true;
	MSpriteArrayIt		it;
	ABint	i = 0;
	
	this->mSpriteArray.resize( mTextLengthMax );

	for( it = this->mSpriteArray.begin() ; it != this->mSpriteArray.end(); it++ ){
		GGSprite*	sprite = &(*it);
		
		sprite->m_width		= this->mWidth;
		sprite->m_height	= this->mHeight;
		sprite->m_locate	= this->mLocate;
		sprite->m_color 	= this->mColor;
		sprite->m_textureName = this->mSurface->getTextureName();
		
		ret = sprite->setup( inIt, inZorder );
		if( ! ret ){
			break;
		}
		++i;
	}
	this->mVisible = true;
	return ret;
}

//-------------------------------------------------------------------------------------------------
//	XvCgN[ibv
//-------------------------------------------------------------------------------------------------
void	GGTextSprite::cleanup( void )
{
	MSpriteArrayIt		it;
	for( it = this->mSpriteArray.begin() ; it != this->mSpriteArray.end(); it++ ){
		GGSprite*	sprite = &(*it);
		sprite->cleanup();
	}
	this->mSpriteArray.clear();
}

//-------------------------------------------------------------------------------------------------
//	`悷iXvCgzu邾j
//-------------------------------------------------------------------------------------------------
ABbool	GGTextSprite::draw( ABint8*  inText )
{
	MSpriteArrayIt		it;
	ABfloat32			x = this->mLocate.x;
	ABuint				i = 0;
	ABint				w, h;
	
	if( this->mVisible ){
		this->mSurface->getTextureSize( &w, &h );

		for( it = this->mSpriteArray.begin() ; it != this->mSpriteArray.end(); it++ ){
			GGSprite*	sprite = &(*it);
			sprite->m_visible = false;
		}
		
		it = this->mSpriteArray.begin();
		while( it != this->mSpriteArray.end() && inText[i] != 0 ){
			GGSprite*	sprite = &(*it);
			
			//	location
			sprite->m_locate.set( x, this->mLocate.y, this->mLocate.z );
			
			//	surface
			GGFont::MFont*		font = this->mFont->getFontTable( inText[ i ] );
			if( font->enable ){
				sprite->m_texCoord[0].x = (ABfloat32)( this->mFont->mSurfaceOffsetX + font->cellX * this->mFont->mCellWidth ) / (ABfloat32)w;
				sprite->m_texCoord[0].y = (ABfloat32)( this->mFont->mSurfaceOffsetY + font->cellY * this->mFont->mCellHeight ) / (ABfloat32)h;
				sprite->m_texCoord[1].x = (ABfloat32)( this->mFont->mSurfaceOffsetX + font->cellX * this->mFont->mCellWidth + this->mFont->mCellWidth) / (ABfloat32)w;
				sprite->m_texCoord[1].y = (ABfloat32)( this->mFont->mSurfaceOffsetY + font->cellY * this->mFont->mCellHeight + this->mFont->mCellHeight ) / (ABfloat32)h;
				sprite->m_visible = true;
			}
			sprite->update();
			
			x += this->mWidth;
			it++;
			++i;
		}
	}

	return true;
}

//-------------------------------------------------------------------------------------------------
//	XvCg擾
//-------------------------------------------------------------------------------------------------
GGSprite*	GGTextSprite::getSprite( ABuint  inNumber )
{
	if( this->mSpriteArray.size() <= inNumber ){
		return NULL;
	}

	return &( this->mSpriteArray[inNumber] );
}

//-------------------------------------------------------------------------------------------------
//	XvCgB
//-------------------------------------------------------------------------------------------------
void	GGTextSprite::setVisible( ABbool  inVisible )
{
	MSpriteArrayIt		it;
	if( ! inVisible ){
		for( it = this->mSpriteArray.begin() ; it != this->mSpriteArray.end(); it++ ){
			it->m_visible = false;
		}
	}
	else{
		for( it = this->mSpriteArray.begin() ; it != this->mSpriteArray.end(); it++ ){
			it->m_visible = true;
		}
	}
	this->mVisible = inVisible;
}

//-------------------------------------------------------------------------------------------------
//	FύX
//-------------------------------------------------------------------------------------------------
void	GGTextSprite::updateColor( void )
{
	MSpriteArrayIt		it;
	for( it = this->mSpriteArray.begin() ; it != this->mSpriteArray.end(); it++ ){
		it->m_color = this->mColor;
	}
}
