/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		GGTexture.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	"OSConfig.h"
#include	"GGTexture.h"
#include	"GameFile.h"

#include	"CWGameMediaService.h"
#include	"GMSGraphicImporter.h"

#include	"GEMacFileInfo.h"

using namespace code_worker;

//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
GGTexture::GGTexture( void )
:
	m_textureNumber( 0 ),
	m_saveDevice( NULL ),
	m_savePort( NULL ),
	m_currentContext( NULL ),
	m_setup( false )
{
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
GGTexture::~GGTexture( void )
{
	this->endDraw();
	this->closeBitmap();
}

//-------------------------------------------------------------------------------------------------
//	摜t@Cǂݍ
//-------------------------------------------------------------------------------------------------
ABbool		GGTexture::opneBitmap(	 	
	const GameFileInfo*				inFileInfo,				//	t@C
	const ABbool					inFixTextureSize,		//	eNX`TCYdli2̗ݏjŐ
	const GEbmpTransparentType		inTrans,				//	ߐF̑I
	const ABint						inWidth,				//	쐬TCY̎w
	const ABint						inHeight				//	쐬TCY̎w
){
	CWresultCode		result = CW_NO_ERROR;
	GMSGraphicImporter	importer;
	CWuint32			width, height;

	FSRef				fsRef = inFileInfo->getFSRef();
	FSSpec				fsSpec;
	OSErr	err = GEMacFileTool::makeFSSpec( &fsRef, &fsSpec );
	if( err != noErr ){
		return	false;
	}
	
	//	eNX`ǂݍ
	result = importer.open( &fsSpec );
	if( result != CW_NO_ERROR){
		printf( "[fsSpec.name:%s]\n", fsSpec.name );
	}
	ABTestReturnReportError( result != CW_NO_ERROR, "importer.open." );

	if( inWidth != 0 ){
		//	摜TCYw莞́Ȃ傫ō쐬
		width = inWidth;
		height = inHeight;
	}
	else{
		importer.getSize( &width, &height );
	}

	//	GWorld쐬
	result = this->m_offScreen.createTexture( width, height, GMS_COLOR_DEPTH_32 );
	ABTestReturnReportError( result != CW_NO_ERROR, "this->m_offScreen.createTexture." );
	
	//	GWrold֕`
	result = importer.draw( this->m_offScreen.getOffScreen() );
	ABTestReturnReportError( result != CW_NO_ERROR, "importer.draw." );

	//	Alpha`l
	this->createAlphaMask( inTrans );

	this->m_setup = true;
	return	true; 
}

//-------------------------------------------------------------------------------------------------
//	摜t@Calpha}XN쐬
//-------------------------------------------------------------------------------------------------
void		GGTexture::createAlphaMask( const GEbmpTransparentType  inTrans )
{
	//	Alpha`l
	switch( inTrans ){
		case GE_BMP_TRANSPARENT_NONE:
			//	alpha`lĂΎIalphaC[gp
		case GE_BMP_TRANSPARENT_ALPHA:	//	alphaC[gpē
			break;
			
		case GE_BMP_TRANSPARENT_BLACK:	//	RGB( 0, 0, 0 )͓ɂȂ܂
			this->m_offScreen.createAlpha( 0x00000000 );
			break;

		case GE_BMP_TRANSPARENT_WHITE:	//	RGB( 255, 255, 255 )͓ɂȂ܂
		default:
			this->m_offScreen.createAlpha( 0x00FFFFFF );
			break;
	}
}

//-------------------------------------------------------------------------------------------------
//	摜t@Cj
//-------------------------------------------------------------------------------------------------
void		GGTexture::closeBitmap( void )
{
	if( this->m_setup && this->m_offScreen.getBuffer() != NULL ){
		this->deleteTexture();
		this->m_offScreen.release();
		this->m_setup = false;
	}
}

//-------------------------------------------------------------------------------------------------
//	̃obt@쐬
//-------------------------------------------------------------------------------------------------
ABbool		GGTexture::createBitmap( const ABint  inWidth, const ABint  inHeight, const GEColorDepthType  inDepth )
{
	//	GWorld쐬
	CWresultCode	result = this->m_offScreen.createTexture( inWidth, inHeight, inDepth );
	if( result != CW_NO_ERROR ){ return false; }
	
	return	true; 
}

//-------------------------------------------------------------------------------------------------
//	obt@֕`s
//-------------------------------------------------------------------------------------------------
CGContextRef		GGTexture::beginDraw( void )
{
	if( this->m_currentContext == NULL ){
		CGrafPtr		port = this->m_offScreen.getOffScreen();
	    GetGWorld( &this->m_savePort, &this->m_saveDevice ); 
		SetGWorld( port, this->m_saveDevice );

		CreateCGContextForPort( port, &this->m_currentContext );		
		QDBeginCGContext( port, &this->m_currentContext );
	}
	return	this->m_currentContext;
}

void		GGTexture::endDraw( void )
{
	if( this->m_currentContext ){
		CGrafPtr		port = this->m_offScreen.getOffScreen();

		QDEndCGContext( port, &this->m_currentContext );
		CGContextRelease( this->m_currentContext );

	    SetGWorld( this->m_savePort, this->m_saveDevice ); 

		this->m_savePort = NULL;
		this->m_saveDevice = NULL;
		this->m_currentContext = NULL;
	}
}
//-------------------------------------------------------------------------------------------------
//	obt@NA
//-------------------------------------------------------------------------------------------------
void		GGTexture::clear( void )
{
	ABuint8*		buffer	= this->m_offScreen.getBuffer();
	GWorldPtr		port	= this->m_offScreen.getOffScreen();
	PixMapHandle	pixmap	= GetGWorldPixMap( port );
	ABint32			row		= GetPixRowBytes(pixmap);
	ABint			Height	= this->m_offScreen.getRealHeight();
	
	PixMapHandle	hPixMap = GetGWorldPixMap( port );
	LockPixels(hPixMap);
	memset( buffer, 0, row * Height * sizeof( ABint8 ) );
	UnlockPixels( hPixMap );
}

//-------------------------------------------------------------------------------------------------
//	obt@XVi݌ʂ𔽉f邽߂Ƀobt@XVKvj
//-------------------------------------------------------------------------------------------------
void		GGTexture::flushBuffer( void )
{
	CGContextFlush( this->m_currentContext );
}

//-------------------------------------------------------------------------------------------------
//	obt@֕`sƂ̐Fݒ
//-------------------------------------------------------------------------------------------------
void		GGTexture::setFillColor( const ABint  inR, const ABint  inG, const ABint  inB, const ABint  inAlpha )
{
	//	set color
	CGContextSetRGBFillColor( this->m_currentContext, (float)inR / 255.0f, (float)inG, (float)inB / 255.0f, (float)inAlpha / 255.0f );
}

//-------------------------------------------------------------------------------------------------
//	obt@擾
//-------------------------------------------------------------------------------------------------
ABuint8*	GGTexture::getBuffer( void )
{
	return	this->m_offScreen.getBuffer();
}

//-------------------------------------------------------------------------------------------------
//	obt@TCY擾
//-------------------------------------------------------------------------------------------------
void		GGTexture::getSize( ABint*  outWidth,  ABint*  outHeight )
{
	*outWidth = this->m_offScreen.getRealWidth();
	*outHeight = this->m_offScreen.getRealHeight();
}

//-------------------------------------------------------------------------------------------------
//	eNX`TCY擾
//-------------------------------------------------------------------------------------------------
void		GGTexture::getTextureSize( ABint*  outWidth,  ABint*  outHeight )
{
	*outWidth = this->m_offScreen.getWidth();
	*outHeight = this->m_offScreen.getHeight();
}

//-------------------------------------------------------------------------------------------------
//	eNX`̃oCh
//-------------------------------------------------------------------------------------------------
ABbool	GGTexture::bindTexture( void )
{
	GLuint		textureName[1];
	CWuint8*	buffer	 = this->m_offScreen.getBuffer();
	CWuint32	width	 = this->m_offScreen.getWidth();
	CWuint32	height	 = this->m_offScreen.getHeight();

	textureName[0] = 0;
	glGenTextures( 1 , textureName );
	if( textureName[0] == 0 ){
		return false;
	}
	
	glPixelStorei( GL_UNPACK_ALIGNMENT, 4 );

	//	Apple@\ggpBGLhCo[ʂĂVRAMԃobt@[Rs[Ƃ΂āA炵
	//	AGPK{ƎvB
	glPixelStorei( GL_UNPACK_CLIENT_STORAGE_APPLE, GL_TRUE );

	//	ŏ̃oCh
	glBindTexture( GL_TEXTURE_2D, textureName[0] ) ;

	//	ݒ
	//		ElinearɂƁATextureɃA`GAVOʂ|AxȂB
	//		@摜ɂĂ͋E悪Ȃ邱ƂB
//	GLint	quality = GL_NEAREST;
	GLint	quality = GL_LINEAR;
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, quality);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, quality);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

	//	Texture`
	glTexImage2D(	GL_TEXTURE_2D, 0,
					GL_RGBA, 
					width, height, 0,
					GL_BGRA_EXT, GL_UNSIGNED_INT_8_8_8_8_REV, buffer );

	this->m_textureNumber = textureName[0];
	return true;
}

//-------------------------------------------------------------------------------------------------
//	eNX`̍XV
//-------------------------------------------------------------------------------------------------
void	GGTexture::updateTexture( void )
{
	ABuint8*	buffer = this->getBuffer();

	glBindTexture( GL_TEXTURE_2D, this->m_textureNumber ) ;
	glTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, this->m_offScreen.getWidth(), this->m_offScreen.getHeight(), 
						GL_BGRA_EXT, GL_UNSIGNED_INT_8_8_8_8_REV, buffer ) ;
}
//-------------------------------------------------------------------------------------------------
//	eNX`̍폜
//-------------------------------------------------------------------------------------------------
void	GGTexture::deleteTexture( void )
{
	GLuint		textureName[1];
	if( this->m_textureNumber != 0 ){
		textureName[0] = this->m_textureNumber;
		glDeleteTextures( 1 , textureName );
		this->m_textureNumber = 0;
	}
}