/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		GameAudio.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

//#define	AB_LOCAL_DEBUG_MODE

#include	"GlobalConfig.h"

#include	<vector>
#include	<list>

#include	"GameAudio.h"
#include	"GameFile.h"

//	CodeWorker
#include	"CWGameAudioService.h"
#include	"GASAudioService.h"
#include	"GGRender.h"
using namespace	code_worker;


//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	INSIDE CLASS
//
//////////////////////////////////////////////////////////////////////////////////////////////////
typedef std::list<ABuint>					GATABuintList;
typedef GATABuintList::iterator				GATABuintListIt;

typedef	struct GATChannelInfo{
	ABuint		channel;
	ABint		count;
	ABint		soundFileNumber;

	GATChannelInfo(){
		channel = 0;
		count = 8;
		soundFileNumber = -1;
	}
	GATChannelInfo( ABuint  inNumber ){
		channel = inNumber;
		count = 8;
		soundFileNumber = -1;
	}
	GATChannelInfo( ABuint  inNumber, ABint  inFileNumber ){
		channel = inNumber;
		count = 8;
		soundFileNumber = inFileNumber;
	}
} GATChannelInfo;
typedef std::list<GATChannelInfo>			GATSoundChannelList;
typedef	GATSoundChannelList::iterator		GATSoundChannelListIt;


//	GASSoundFileRef
typedef	struct	GATSoundFileInfo{
	GASSoundFileRef			soundFile;
	ABint					channel;
	GATSoundChannelListIt	channelIt;
	
	GATSoundFileInfo( GASSoundFileRef  inRef = NULL, ABint  inChannel = -1 ){
		channel = inChannel;
		soundFile = inRef;
	}
} GATSoundFileInfo;

typedef std::vector<GATSoundFileInfo>		GATSoundFileArray;
typedef	GATSoundFileArray::iterator		GATSoundFileArrayIt;

//	`lz
class	GATrackMultiChannelTrack{
public:
	GATSoundChannelList	mUseChannelList;
	GATABuintListIt		mPlayMultiChannelTrackListIt;

	GATrackMultiChannelTrack(){
	};
	~GATrackMultiChannelTrack(){
	};
};

typedef std::vector<GATrackMultiChannelTrack>		GATrackMultiChannelTrackArray;
typedef	GATrackMultiChannelTrackArray::iterator	GATrackMultiChannelTrackArrayIt;

typedef GATABuintList								GATrackMultiChannelTrackItList;
typedef	GATABuintListIt							GATrackMultiChannelTrackItListIt;

//	GASAudioServicef[^JvZ
class	GameAudioPlayer{
public:
	GASAudioEffect		mEffect;
	GASConfiguration	mConfig;
	GASAudioService		mAudioPlayer;
	
	GATSoundFileArray	mSoundFileArray;	//	TEht@CǗz
	GEUniqueNumber		mSoundFileNumber;	//	gpt@CBpNumber

	GATrackMultiChannelTrackArray	mMultiChannelTrackArray;		//	multichannelgbNǗz

	GATrackMultiChannelTrackItList	mPlayMultiChannelTrackList;		//	ĐmultichannelgbNǗXg
	GEUniqueNumber		mSoundChannelNumber;						//	gp`lBpNumber

	GATrackMultiChannelTrack		mSoundNumberTrack;				//	GE_AUDIO_SOUND_TRACK_SOUNDNUMBER_CHANNELp

public:
	GameAudioPlayer();
	~GameAudioPlayer();
};

#define	GE_AUDIO_SOUND_MASTER_VOLUME	(1.00f)		

//	e`l̉ʁBMaster{[Ƃ͕ʂɂ܂B
//	0.0-1.0f̊ԂłABMS̉͗]悭Ȃ̂ŁAꂵ₷łB7x傤ǂ悢悤łB
#define	GE_AUDIO_SOUND_VOLUME_VALANCE(x)	( 0.75f * x )		

#define	GATGetMultiChannelNumber( x )		( GE_AUDIO_SOUND_SINGLE_CHANNEL_TRACK_COUNT + x )
#define	GATGetChannelTotalCount			( GE_AUDIO_SOUND_CHANNEL_COUNT )
#define	GATGetSoundObject(x)			(GASSoundObjectRef)( x + 1 )

//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
GameAudioPlayer::GameAudioPlayer( void )
{
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
GameAudioPlayer::~GameAudioPlayer( void )
{
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CLASS	GameAudio
//
//////////////////////////////////////////////////////////////////////////////////////////////////
ABfloat32		GameAudio::m_masterVolume = GE_AUDIO_SOUND_MASTER_VOLUME;

//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
GameAudio::GameAudio( void )
{
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
GameAudio::~GameAudio( void )
{
	this->finalize();
}

//-------------------------------------------------------------------------------------------------
//	Initialize
//-------------------------------------------------------------------------------------------------
ABbool	GameAudio::initialize( ABbool  inUserReverb, CWsize  inCacheFileCount, CWsize  inMemorySize, GASsampleRate  inInputSampleRate, GASsampleChannelType  inChannelType )
{
	this->finalize();
	
	if( inInputSampleRate < GAS_SAMPLE_RATE_05012 ){
		return	false;
	}
	
	try{
		this->m_soundTrack = new	GameAudioPlayer;
	}
	catch( ... ){
		ABTestReturnReportError( true, "bad alloc." )
	}
	
	CWresultCode		result;
	GASAudioEffect*		effect = NULL;
	
	this->m_volumeBalance = 1.0f;

	//	o[uݒ
	this->m_soundTrack->mEffect.reverbQualyty			= GAS_RENDER_QUALITY_MIN;			//	
	this->m_soundTrack->mEffect.reverbType				= GAS_EFFECT_REVERB_TYPE_LARGE_ROOM;	//	Reverb̎
	this->m_soundTrack->mEffect.reverbParamDryWetMix	= 20.0f;								//	o[ûi hC/EFbg̔䗦wB j ( 0.0 - 100.0f )

	this->m_soundTrack->mConfig.audioSampleRate				= inInputSampleRate;				//	Ŝ̉
	this->m_soundTrack->mConfig.audioFileBufferTotalSize	= inMemorySize * inCacheFileCount;	//	I[fBIt@C[ɓǂݍލۂ̑TCY
	this->m_soundTrack->mConfig.audioFileBufferSize			= inMemorySize;						//	I[fBIt@C[ɓǂݍލۂ̃t@C̃TCY
	this->m_soundTrack->mConfig.audioBusCount				= GATGetChannelTotalCount;			//	ɍĐ鉹̍ő吔
	this->m_soundTrack->mConfig.audioInputChannel			= inChannelType;					//	͂XeI܂̓mɂ
	this->m_soundTrack->mConfig.audioOutputChannel			= GAS_SAMPLE_CHANNEL_STEREO																				;			//	ŏIo͂XeI܂̓mɂ

	if( inUserReverb ){
		effect = &this->m_soundTrack->mEffect;
	}

	//		
	//	::TODO:: 65ȏw肷ƁANbV
	result = this->m_soundTrack->mAudioPlayer.initialize( &this->m_soundTrack->mConfig, effect );
	ABTestReturnReportError( result != CW_NO_ERROR, "error GASAudioService init." )
	
	//	}``lgbN
	this->m_soundTrack->mMultiChannelTrackArray.resize( GE_AUDIO_SOUND_MULTI_CHANNEL_TRACK_COUNT );
	
	GASSoundObjectRef	soundObject;
	CWint	i;
	for( i = 0 ; i < GATGetChannelTotalCount; i++ ){
		//	soundObject1`nBPɕKvAȌ͔ԍ+1Ďgp
		soundObject = this->m_soundTrack->mAudioPlayer.createSoundObject();
		ABTestReturnReportError( soundObject == 0, "error createSoundObject." )
	}
	
	this->setMasterVolume( GameAudio::m_masterVolume );
	
	return true;
}

//-------------------------------------------------------------------------------------------------
//	Finalize
//-------------------------------------------------------------------------------------------------
void	GameAudio::finalize( void )
{
	if( this->m_soundTrack ){
		this->m_soundTrack->mAudioPlayer.setMasterVolume( 0.0f );
		ABint	i;
		for( i = 0 ; i < GE_AUDIO_SOUND_SINGLE_CHANNEL_TRACK_COUNT; i ++ ){
			this->stopSoundTrack( GE_AUDIO_SOUND_TRACK_SINGLE_CHANNEL, i );
		}
		for( i = 0 ; i < GE_AUDIO_SOUND_MULTI_CHANNEL_TRACK_COUNT; i ++ ){
			this->stopSoundTrack( GE_AUDIO_SOUND_TRACK_MULTI_CHANNEL, i );
		}
		for( i = 0 ; i < this->m_soundTrack->mSoundFileArray.size(); i ++ ){
			this->closeSoundFile( i );
		}
	
		this->m_soundTrack->mAudioPlayer.finalize();
		delete	this->m_soundTrack;
		this->m_soundTrack = NULL;
	}
}

//-------------------------------------------------------------------------------------------------
//	
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
//	Master{[
//-------------------------------------------------------------------------------------------------
void	GameAudio::setMasterVolume( ABfloat32  inVol )
{
	GameAudio::m_masterVolume = inVol;
	if( this->m_soundTrack ){
		this->m_soundTrack->mAudioPlayer.setMasterVolume( inVol );
	}
}

//-------------------------------------------------------------------------------------------------
//	Đʎw( 0.0 - 1.0 )
//-------------------------------------------------------------------------------------------------
void	GameAudio::setVolumeBalance( ABfloat32  inVol )
{
	this->m_volumeBalance = inVol;
}

//-------------------------------------------------------------------------------------------------
//	SgbN~
//-------------------------------------------------------------------------------------------------
void	GameAudio::stopAllSound( void )
{
}

//-------------------------------------------------------------------------------------------------
//	gbN
//-------------------------------------------------------------------------------------------------
void	GameAudio::setSoundTrackVolume( void )
{
}

//-------------------------------------------------------------------------------------------------
//	Đ
//-------------------------------------------------------------------------------------------------
ABbool	GameAudio::playSoundTrack( GEAudioSoundTravkType  inType, ABuint  inTrackNumber, ABuint  inFileNumber )
{
	ABbool	ret = false;
	CWresultCode	result;
	
	ABTestReturnReportError( this->m_soundTrack->mSoundFileArray.size() <= inFileNumber, "faital argument inFileNumber." );

	switch( inType ){
		case GE_AUDIO_SOUND_TRACK_SINGLE_CHANNEL:
			{
				GATSoundFileInfo		fileInfo =  this->m_soundTrack->mSoundFileArray[ inFileNumber ];
				result = this->m_soundTrack->mAudioPlayer.setVolume( GATGetSoundObject(inTrackNumber), GE_AUDIO_SOUND_VOLUME_VALANCE( this->m_volumeBalance ) );
				result = this->m_soundTrack->mAudioPlayer.playWave( GATGetSoundObject(inTrackNumber), fileInfo.soundFile );
				ABTestBreakReportError( result != CW_NO_ERROR, "playWave." )
				ret = result == CW_NO_ERROR;
			}
			break;

		case GE_AUDIO_SOUND_TRACK_MULTI_CHANNEL:
			{
		
				ABuint		channelNumber;

				//	V`l
				if( GE_AUDIO_SOUND_MULTI_CHANNEL_TRACK_COUNT <= this->m_soundTrack->mSoundChannelNumber.getUseCount() ){

					//	g؂Ă̂ŁAÂ`lėp
					if( this->m_soundTrack->mPlayMultiChannelTrackList.empty() ){
						//	ĝ
						ret = false;
						break;
					}

					//	ƂÂĐgbNQ
					ABuint		trackNumber;
					trackNumber = this->m_soundTrack->mPlayMultiChannelTrackList.front();
					this->m_soundTrack->mPlayMultiChannelTrackList.pop_front();

					//	̃gbNĐĂ`lQ
					GATrackMultiChannelTrack*	track = &( this->m_soundTrack->mMultiChannelTrackArray[trackNumber] );
					GATSoundChannelListIt		cit;
					GATChannelInfo				channelInfo = track->mUseChannelList.front();
					channelNumber = channelInfo.channel;
					track->mUseChannelList.pop_front();
					
					if( ! track->mUseChannelList.empty() ){
						//	gbN`lۗLĂƂ́AXgɖ߂
						this->m_soundTrack->mPlayMultiChannelTrackList.push_back( trackNumber );
						track->mPlayMultiChannelTrackListIt = this->m_soundTrack->mPlayMultiChannelTrackList.end();
						--track->mPlayMultiChannelTrackListIt;
					}
					else{
						//	gbN`lȂȂAXg菜̂ŁAȂ
					}
					
					ABLocalDebug( printf( "Recycle channel [%d] \n" , channelNumber ) );
					
				}
				else{

					channelNumber = this->m_soundTrack->mSoundChannelNumber.getNewNumber();

					GATrackMultiChannelTrack*	track = &( this->m_soundTrack->mMultiChannelTrackArray[ inTrackNumber ] );
					track->mUseChannelList.push_back( GATChannelInfo(channelNumber) );
					this->m_soundTrack->mPlayMultiChannelTrackList.push_back( inTrackNumber );
					track->mPlayMultiChannelTrackListIt = this->m_soundTrack->mPlayMultiChannelTrackList.end();
					--track->mPlayMultiChannelTrackListIt;
				}

				GATSoundFileInfo		fileInfo =  this->m_soundTrack->mSoundFileArray[ inFileNumber ];
				result = this->m_soundTrack->mAudioPlayer.setVolume( GATGetSoundObject(GATGetMultiChannelNumber( channelNumber )), GE_AUDIO_SOUND_VOLUME_VALANCE( this->m_volumeBalance ) );
				result = this->m_soundTrack->mAudioPlayer.playWave( GATGetSoundObject(GATGetMultiChannelNumber( channelNumber )), fileInfo.soundFile );
				ABTestBreakReportError(result != CW_NO_ERROR, "playWave." )
				ret = result == CW_NO_ERROR;
			}
			break;
			
		case GE_AUDIO_SOUND_TRACK_SOUNDNUMBER_CHANNEL:
			{
			
				//	̏ꍇgbNԍ͖
				ABuint		channelNumber;
				GATSoundFileInfo*		fileInfo =  &( this->m_soundTrack->mSoundFileArray[ inFileNumber ] );
				//	gĂ邩H
				if( fileInfo->channel != -1 ){
					//	gĂƂ͍ėp
//					ret = this->m_soundTrack->mAudioPlayer.stopWave( GATGetMultiChannelNumber( fileInfo->channel ) );
//					ABTestBreakReportError( !ret, "stopWave." )
				}
				else{
					GATChannelInfo		channelInfo;
					if( GE_AUDIO_SOUND_MULTI_CHANNEL_TRACK_COUNT <= this->m_soundTrack->mSoundChannelNumber.getUseCount() ){
					
						//	󂫂
						channelInfo = this->m_soundTrack->mSoundNumberTrack.mUseChannelList.front();
						this->m_soundTrack->mSoundNumberTrack.mUseChannelList.pop_front();
						fileInfo->channelIt = this->m_soundTrack->mSoundNumberTrack.mUseChannelList.end();

						channelInfo.count = 8;
						channelInfo.soundFileNumber = inFileNumber;
						channelNumber = channelInfo.channel;

						ABLocalDebug( printf( "Recycle channel [%d] \n" , channelInfo.channel ) );
					}
					else{
						//	V`l
						channelNumber = this->m_soundTrack->mSoundChannelNumber.getNewNumber();
					}
					
					channelInfo.channel = channelNumber;
					channelInfo.soundFileNumber = inFileNumber;
					this->m_soundTrack->mSoundNumberTrack.mUseChannelList.push_back( channelInfo );

					fileInfo->channel = channelNumber;
					fileInfo->channelIt = this->m_soundTrack->mSoundNumberTrack.mUseChannelList.end();
					--fileInfo->channelIt;
				}
				result = this->m_soundTrack->mAudioPlayer.setVolume( GATGetSoundObject(GATGetMultiChannelNumber( fileInfo->channel )), GE_AUDIO_SOUND_VOLUME_VALANCE( this->m_volumeBalance ) );
				result = this->m_soundTrack->mAudioPlayer.playWave( GATGetSoundObject(GATGetMultiChannelNumber( fileInfo->channel )), fileInfo->soundFile );
				ABTestBreakReportError( result != CW_NO_ERROR, "playWave." )
				ret = result == CW_NO_ERROR;
				ABLocalDebug( printf( "use channel count [%d] \n" , this->m_soundTrack->mSoundChannelNumber.getUseCount() ) );
			}
			break;
		default:
			ABReport( "Unkown type." );
			break;
	}

	return	ret;
}
//-------------------------------------------------------------------------------------------------
//	~
//-------------------------------------------------------------------------------------------------
ABbool	GameAudio::stopSoundTrack( GEAudioSoundTravkType  inType, ABuint  inTrackNumber )
{
	ABbool	ret = false;
	
	switch( inType ){
		case GE_AUDIO_SOUND_TRACK_SINGLE_CHANNEL:
			if( this->m_soundTrack->mAudioPlayer.isPlayWave( GATGetSoundObject(inTrackNumber )) ){
				ret = this->m_soundTrack->mAudioPlayer.stopWave( GATGetSoundObject(inTrackNumber ));
				ABTestBreakReportError( !ret, "playWave." )
			}
			break;

		case GE_AUDIO_SOUND_TRACK_MULTI_CHANNEL:
		case GE_AUDIO_SOUND_TRACK_SOUNDNUMBER_CHANNEL:
			{
				ABuint		channelNumber;

				if( this->m_soundTrack->mMultiChannelTrackArray.empty() ){
					break;
				}

				//	ĐĂ`lXgbv
				GATrackMultiChannelTrack*	track = &( this->m_soundTrack->mMultiChannelTrackArray[ inTrackNumber ] );
				if( ! track->mUseChannelList.empty() ){
					GATChannelInfo		channelInfo = track->mUseChannelList.front();
					channelNumber = channelInfo.channel;

					ret = this->m_soundTrack->mAudioPlayer.stopWave( GATGetSoundObject(GATGetMultiChannelNumber( channelNumber )) );
					this->m_soundTrack->mSoundChannelNumber.releaseNumber( channelNumber );

					track->mUseChannelList.pop_front();
					ABTestBreakReportError( !ret, "stopWave." )
					
					if( track->mUseChannelList.empty() ){
						//	ɂȂAXg폜
						this->m_soundTrack->mPlayMultiChannelTrackList.erase( track->mPlayMultiChannelTrackListIt );
					}
				}
			}
			break;
			
		default:
			ABReport( "Unkown type." );
			break;
	}

	return	ret;
}

//-------------------------------------------------------------------------------------------------
//	Ǘ
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
//	TEht@CJ
//-------------------------------------------------------------------------------------------------
ABbool	GameAudio::openSoundFile( GameFileInfo*  inFile, ABuint*  outFileNumber )
{
	FSRef	fsRef = inFile->getFSRef();
	ABuint8	fila[256];
	inFile->copyPath( fila, 256 );
	ABReportFormat( "OpenWAV [%s]\n", fila );
	GASSoundFileRef		soundFileRef = this->m_soundTrack->mAudioPlayer.openWave( fsRef, (this->m_soundTrack->mConfig.audioFileBufferTotalSize != 0) );
	ABTestReturnReportError( soundFileRef == NULL, "error openWave." )

	//	󂫔z擾
	ABint	number = 	this->m_soundTrack->mSoundFileNumber.getNewNumber();
	if( this->m_soundTrack->mSoundFileArray.size() <= number ){
		this->m_soundTrack->mSoundFileArray.resize( number + 1 );
	}
	GATSoundFileInfo		fileInfo( soundFileRef );
	this->m_soundTrack->mSoundFileArray[ number ] = fileInfo;
	*outFileNumber =  number;
	return	true;
}

//-------------------------------------------------------------------------------------------------
//	TEht@C[CACHE
//-------------------------------------------------------------------------------------------------
ABbool	GameAudio::cashingSoundFile( ABuint  inFileNumber )
{
	if( this->m_soundTrack == NULL ){
		return	false;
	}
	if( inFileNumber < this->m_soundTrack->mSoundFileArray.size() ){

		GATSoundFileInfo		fileInfo =  this->m_soundTrack->mSoundFileArray[ inFileNumber ];
		
		//	::TODO::TCY擾łȂ
		CWsize	size = this->m_soundTrack->mAudioPlayer.getBufferSize( fileInfo.soundFile );
		if( size == 0){
			//	::TODO::this->m_soundTrack->mConfig.audioFileBufferTotalSizeł̓[hłȂ
			CWresultCode	ret = this->m_soundTrack->mAudioPlayer.loadWave( fileInfo.soundFile, this->m_soundTrack->mConfig.audioFileBufferTotalSize );
			ABTestReturnReportError( ret != CW_NO_ERROR, "error loadWave." )
			return	ret == CW_NO_ERROR;
		}
		else{
			ABReportFormat( "size [%d]\n", size );
		}
	}
	return	false;
}

//-------------------------------------------------------------------------------------------------
//	TEht@C
//-------------------------------------------------------------------------------------------------
void	GameAudio::closeSoundFile( ABuint  inFileNumber )
{
	if( this->m_soundTrack == NULL ){
		return	;
	}
	if( inFileNumber < this->m_soundTrack->mSoundFileArray.size() ){
		GATSoundFileInfo		fileInfo =  this->m_soundTrack->mSoundFileArray[ inFileNumber ];
		if( fileInfo.soundFile ){
			if( fileInfo.channel != -1 ){
				while( this->m_soundTrack->mAudioPlayer.isPlayWave( GATGetSoundObject(GATGetMultiChannelNumber( fileInfo.channel ) ) ) ){
					this->m_soundTrack->mAudioPlayer.stopWave( GATGetSoundObject(GATGetMultiChannelNumber( fileInfo.channel ) ) );
				}
				this->stopSoundTrack( GE_AUDIO_SOUND_TRACK_SOUNDNUMBER_CHANNEL, fileInfo.channel );
			}

			this->m_soundTrack->mAudioPlayer.closeWave( fileInfo.soundFile );
			this->m_soundTrack->mSoundFileNumber.releaseNumber( inFileNumber );

			ABLocalDebug( printf( "closeSoundFile %d \n" , inFileNumber ) );
		}
		GATSoundFileInfo	def( 0, -1 );
		this->m_soundTrack->mSoundFileArray[ inFileNumber ] = def;
	}
}

//-------------------------------------------------------------------------------------------------
//	MIDIt@C
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
ABbool	GameAudio::openSMF( GameFileInfo*  inFile )
{
	return	true;
}
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
void	GameAudio::closeSMF( void )
{
}
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
void	GameAudio::playSMF( void )
{
}
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
void	GameAudio::stopSMF( void )
{
}

//-------------------------------------------------------------------------------------------------
//	
//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------
//	TEhnhO
//-------------------------------------------------------------------------------------------------
void	GameAudio::handling( void )
{
	if( this->m_soundTrack == NULL ){
		return;
	}

	//	multichannelgbN̎gp`l
	GATrackMultiChannelTrackItListIt	it;
	for( it = this->m_soundTrack->mPlayMultiChannelTrackList.begin(); it != this->m_soundTrack->mPlayMultiChannelTrackList.end(); it++ ){
		
		GATrackMultiChannelTrack*	track = &( this->m_soundTrack->mMultiChannelTrackArray[ *it ] );
		
		GATSoundChannelListIt	cit;
		if( ! track->mUseChannelList.empty() ){
			for( cit = track->mUseChannelList.begin(); cit != track->mUseChannelList.end(); cit++ ){
				GATChannelInfo		channelInfo = *cit;
				if( ! this->m_soundTrack->mAudioPlayer.isPlayWave( GATGetSoundObject(GATGetMultiChannelNumber( channelInfo.channel) ) ) ){
					--cit->count;
					if( cit->count <= 0  ){
						//	IĂ̂ō폜
						this->m_soundTrack->mSoundChannelNumber.releaseNumber(channelInfo.channel );
						//ABLocalDebug( printf( "Recycle channel %d \n" , channelInfo.channel ) );
						track->mUseChannelList.erase( cit );
					}
				}
			}
			if( track->mUseChannelList.empty() ){
				//	Xg폜
				it = this->m_soundTrack->mPlayMultiChannelTrackList.erase( track->mPlayMultiChannelTrackListIt );
			}
		}
	}

	{
		GATSoundChannelListIt	cit;
		for( cit =  this->m_soundTrack->mSoundNumberTrack.mUseChannelList.begin(); cit !=  this->m_soundTrack->mSoundNumberTrack.mUseChannelList.end(); cit++ ){

			GATChannelInfo		channelInfo = *cit;
			
			if( ! this->m_soundTrack->mAudioPlayer.isPlayWave( GATGetSoundObject(GATGetMultiChannelNumber( channelInfo.channel )) ) ){
				--cit->count;
				if( cit->count <= 0  ){
					//	IĂ̂ō폜
					this->m_soundTrack->mSoundChannelNumber.releaseNumber(channelInfo.channel );
					
					GATSoundFileInfo		fileInfo =  this->m_soundTrack->mSoundFileArray[ channelInfo.soundFileNumber ];
					ABLocalDebug( printf( "Recycle channel %d \n" , channelInfo.channel ) );
					cit = this->m_soundTrack->mSoundNumberTrack.mUseChannelList.erase( cit );

					this->m_soundTrack->mSoundFileArray[ channelInfo.soundFileNumber ].channel = -1;
				}
			}
		}
	}
	
}
