/*///////////////////////////////////////////////////////////////////////////////

---------------------------------------------------------------------------------

	Component:	
	File:		GEMacFileInfo.cpp

	Encoding:	Shift-JIS

	Version:	
		version 1.00	2003/04/13	Reno

---------------------------------------------------------------------------------

	GNU General Public License
	<Aqua'n-Bests>
    Copyright (C) <2004>  <Aqua'n-Bests developer>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*///////////////////////////////////////////////////////////////////////////////

#include	<stdio.h>

#include	"GEMacFileInfo.h"


//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	Class GEMacFileTool
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------------------------------------
//	Constructor
//-------------------------------------------------------------------------------------------------
GEMacFileTool::GEMacFileTool( void )
{
}

//-------------------------------------------------------------------------------------------------
//	Destructor
//-------------------------------------------------------------------------------------------------
GEMacFileTool::~GEMacFileTool( void )
{
}

//-------------------------------------------------------
//	get dir items
//-------------------------------------------------------
OSErr	GEMacFileTool::collectDirectoryItems
(
	const FSRef				*container,
	GameFileInfoList*		fsInfoArray,
	int						maxSize,
	bool					*containerChanged,
	bool					directoryOnly
)
{
	/* Grab items 10 at a time. */
	enum { kMaxItemsPerBulkCall = 10 };
	
	OSErr		result;
	FSIterator	it;
	FSRef		refs[kMaxItemsPerBulkCall];
	ItemCount	actualObjects;
	Boolean		changed;
	unsigned int			i;
	
	FSCatalogInfo		catalogInfo;
	FSCatalogInfoBitmap whichInfo;
	bool		getFlag;

	//	Folder̂ݎ擾@\ǉ
	whichInfo = kFSCatInfoFinderXInfo | kFSCatInfoNodeFlags;

	/* check parameters */
	require_action(  (NULL != containerChanged), BadParameter, result = paramErr);
	
	*containerChanged = false;
	
	/* open an FSIterator */
	result = FSOpenIterator(container, kFSIterateFlat, &it);
	require_noerr(result, FSOpenIterator);
	
	/* Call FSGetCatalogInfoBulk in loop to get all items in the container */
	do
	{
		result = FSGetCatalogInfoBulk(it, kMaxItemsPerBulkCall, &actualObjects,
					&changed, kFSCatInfoNone,  NULL,  refs, NULL, NULL);
		
		/* if the container changed, set containerChanged for output, but keep going */
		if ( changed )
		{
			*containerChanged = changed;
		}
		
		/* any result other than noErr and errFSNoMoreItems is serious */
		require((noErr == result) || (errFSNoMoreItems == result), FSGetCatalogInfoBulk);
		
		/* add objects to output array and count */
		if ( 0 != actualObjects )
		{
			/* concatenate the FSRefs to the end of the	 handle */
			for( i = 0 ; i < actualObjects ; i ++ ){
				//	get folder only ?
				result = FSGetCatalogInfo( &refs[i], whichInfo, &catalogInfo, NULL, NULL, NULL);
				require_noerr(result, CreateFSSpecError );
			
				getFlag = true;
				if( directoryOnly ){
					if( ( catalogInfo.nodeFlags & kFSNodeIsDirectoryMask) == 0 ){
						getFlag = false;
					}
				}
				if( getFlag  ){
					fsInfoArray->push_back( GameFileInfo( refs[i] ) );
/*					
					ABuint8	c[1024];
					getPathFromFSRef( &refs[i], c, 1024 );
					printf( "%s\n", c );
					require( fsInfoArray->size() <= maxSize, FSGetCatalogInfoBulk);
*/
				}
			}
		}
	} while ( noErr == result );
/*
	printf( "\n");
	for( GameFileInfoListIt it = fsInfoArray->begin(); it != fsInfoArray->end(); it++ ){
		ABuint8	c[1024];
		FSRef	fsref =it->getFSRef();
		getPathFromFSRef( &fsref, c, 1024 );
		printf( "%s\n", c );
	}
*/
ArraySizeOver:

	verify_noerr(FSCloseIterator(it)); /* closing an open iterator should never fail, but... */
	
	return ( noErr );
	
	/**********************/

CreateFSSpecError:
FSGetCatalogInfoBulk:

	/* close the iterator */
	verify_noerr(FSCloseIterator(it));

FSOpenIterator:
	/* dispose of handle if already allocated and clear the outputs */
	
BadParameter:

	return ( result );
}

//-------------------------------------------------------
//	fullpath -> FSRef
//-------------------------------------------------------
bool	GEMacFileTool::getFSRefFromPath
(
	UInt8 	*path,
	FSRef	*fileRefPtr,
	bool	*isDirectory
)
{
	OSStatus	result;
	Boolean 	flagDirectory;
	
	/* check parameters */
	if( NULL == fileRefPtr ){
		return	false;
	}
	
	/* convert the POSIX path to an FSRef */
	result = FSPathMakeRef( path, fileRefPtr, &flagDirectory );
	if( result != noErr ){
		return	false;
	}
	
	if( flagDirectory == TRUE ){
		*isDirectory = true;
	}else{
		*isDirectory = false;
	}
	
	return	true;
}

//-------------------------------------------------------
//	FSRef -> fullpath
//-------------------------------------------------------
bool	GEMacFileTool::getPathFromFSRef
(
	FSRef	*fileRefPtr,
	UInt8*	path,
	UInt32 	maxPathSize
)
{
	OSStatus	result;
	
	/* and then convert the FSRef to a path */
	result = FSRefMakePath( fileRefPtr, path, maxPathSize );
	if( result != noErr ){
		return	false;
	}
	return	true;
}


//-------------------------------------------------------
//	fullpath -> FSSpec
//-------------------------------------------------------
bool	GEMacFileTool::getFSSpecFromPath
(
	UInt8 	*path,
	FSSpec	*fileSpecPtr,
	bool	*isDirectory
)
{
	OSStatus	result;
	FSRef		ref;
	Boolean 	flagDirectory;
	
	/* check parameters */
	if( NULL == fileSpecPtr ){
		return	false;
	}
	
	/* convert the POSIX path to an FSRef */
	result = FSPathMakeRef( path, &ref, &flagDirectory );
	if( result != noErr ){
		return	false;
	}
	
	/* and then convert the FSRef to an FSSpec */
	result = FSGetCatalogInfo( &ref, kFSCatInfoNone, NULL, NULL, fileSpecPtr, NULL );
	if( result != noErr ){
		return	false;
	}

	if( flagDirectory == TRUE ){
		*isDirectory = true;
	}else{
		*isDirectory = false;
	}
	
	return	true;
}

//-------------------------------------------------------
//	FSSpec -> fullpath
//-------------------------------------------------------
bool	GEMacFileTool::getPathFromFSSpec
(
	FSSpec*	fileSpecPtr,
	UInt8*	path,
	UInt32 	maxPathSize
)
{
	OSStatus	result;
	FSRef		ref;
	
	/* convert the inputs to an FSRef */
	result = GEMacFileTool::makeFSRef(fileSpecPtr->vRefNum, fileSpecPtr->parID, fileSpecPtr->name, &ref);
	if( result != noErr ){
		return	false;
	}
	
	/* and then convert the FSRef to a path */
	result = FSRefMakePath(&ref, path, maxPathSize);
	if( result != noErr ){
		return	false;
	}
	return	true;
}

//-------------------------------------------------------
//	FSSpec -> FSRef
//-------------------------------------------------------
OSErr	GEMacFileTool::getFSRefFromFSSpec(
	FSSpec*	fileSpecPtr,
	FSRef *ref 
)
{
	return	GEMacFileTool::makeFSRef(fileSpecPtr->vRefNum, fileSpecPtr->parID, fileSpecPtr->name, ref);
}

//-------------------------------------------------------
//	makeFSRef
//-------------------------------------------------------
OSErr	GEMacFileTool::makeFSRef
(
	FSVolumeRefNum		volRefNum,
	SInt32				dirID,
	ConstStr255Param	name,
	FSRef *ref)
{
	OSErr		result;
	FSRefParam	pb;
	
	/* check parameters */
	require_action(NULL != ref, BadParameter, result = paramErr);
	
	pb.ioVRefNum = volRefNum;
	pb.ioDirID = dirID;
	pb.ioNamePtr = (StringPtr)name;
	pb.newRef = ref;
	result = PBMakeFSRefSync(&pb);
	require_noerr(result, PBMakeFSRefSync);
	
PBMakeFSRefSync:
BadParameter:

	return ( result );
}

//-------------------------------------------------------
//	makeFSSpec
//-------------------------------------------------------
OSErr	GEMacFileTool::makeFSSpec(
	const FSRef *ref,
	FSSpec *spec)
{
	OSErr	result;
	
	/* check parameters */
	require_action(NULL != spec, BadParameter, result = paramErr);
	
	result = FSGetCatalogInfo( ref, kFSCatInfoNone, NULL, NULL, spec, NULL );
	require_noerr(result, FSGetCatalogInfo);
	
FSGetCatalogInfo:
BadParameter:

	return ( result );
}

