//	GVSView

#include <Carbon/Carbon.h>
#include <stdio.h>

#include	"CWorkerConfig.h"
#include	"CWorkerDevTool.h"
#include	"GUIView.h"

using namespace code_worker;

GUIView::GUIView( void )
{
	this->m_switchingScreenMode = false;
	this->m_guiapp = NULL;
}
GUIView::~GUIView( void )
{
	this->finalize();
}

//------------------------------------------------------------------------------
//	initialize
//------------------------------------------------------------------------------
CWbool	GUIView::initialize( void )
{
	CWresultCode			result;
	GLSContextViewOption	option;
	CWint16					width, height;

	this->getSize( &width, &height );

	GLSGetDefaultContextViewOpetion( option );
	option.width		= width;
	option.height		= height;

	result = this->m_glView.initialize( &option );
	if( result != CW_NO_ERROR ){ return	false; }

	//	̃EBhEɕ`悷
	result = this->m_glView.attachWindow( GetWindowPort(this->getWindowRef()) );
	if( result != CW_NO_ERROR ){ return	false; }
	
	result = this->m_glView.getCurrentContext()->setupFont( "Genova", 10 );
	if( result != CW_NO_ERROR ){ return	false; }

	this->repositionWindow( GVS_WINDOW_POSITION_CENTER_ON_MAIN_SCREEN );
	this->show();
	
	return	this->onInit();
}
//------------------------------------------------------------------------------
//	finalize
//------------------------------------------------------------------------------
void	GUIView::finalize( void )
{
	this->onFinal();
	this->m_glView.finalize();
}

//------------------------------------------------------------------------------
//	̕`
//------------------------------------------------------------------------------
void	GUIView::drawInfo( void )
{
	GLSContext*  context = this->m_glView.getCurrentContext();
	Rect		rectPort;
	CGrafPtr	port = aglGetDrawable( context->getContext());
	GetPortBounds( port, &rectPort );
	GLfloat		width = rectPort.right - rectPort.left;
	GLfloat		height = rectPort.bottom - rectPort.top;

	//	t[[g
	static CWint8				frameRate[256] = "";
	static CWint32	 			frames = 0;
	static AbsoluteTime 	time = {0,0};
	AbsoluteTime			currTime = UpTime ();

	double		deltaTime = (float) AbsoluteDeltaToDuration (currTime, time);
	if( 0 > deltaTime ){
		// if negative microseconds
		deltaTime /= -1000000.0;
	}
	else{
		// else milliseconds
		deltaTime /= 1000.0;
	}
	frames++;

	if( 0.5 <= deltaTime ){
	// has update interval passed
		sprintf( frameRate, "FPS: %0.1f",  (CWfloat64) frames / deltaTime);
		time = currTime;	// reset for next time interval
		frames = 0;
	}

	//	`
	glPushAttrib( GL_ALL_ATTRIB_BITS );
	glEnable( GL_BLEND );
	glBlendFunc( GL_SRC_ALPHA ,GL_ONE_MINUS_SRC_ALPHA  );
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();

		glLoadIdentity();

		glMatrixMode( GL_MODELVIEW );
		glPushMatrix();

			glLoadIdentity();
			glScalef( 2.0f / width, -2.0f /  height, 1.0f);
			glTranslatef( -width / 2.0f, -height / 2.0f, 0.0f);

			glColor4f(  0.0f, 0.0f, 0.0f , 0.8f  );
			glBegin(GL_QUADS); 
				glVertex3f(   0, height - 44, 0.0);
				glVertex3f(   0, height, 0.0);
				glVertex3f( 240, height, 0.0);
				glVertex3f( 240, height - 44, 0.0);
			glEnd();

			glColor4f(  1.0f, 1.0f, 1.0f , 1.0f  );
			glRasterPos3f( 16, height - 28, 0); 
			context->drawCString( frameRate );
			glRasterPos3f( 16, height - 16, 0); 
			context->drawCString( (char*) glGetString (GL_VENDOR ) );
			glRasterPos3f( 16, height - 4, 0); 
			context->drawCString( (char*) glGetString (GL_RENDERER) );
			
			this->onDrawInfo( context, &rectPort );

		glPopMatrix();

	glPopMatrix();
	glPopAttrib();
}

//------------------------------------------------------------------------------
//	`ƃ_[ƃXbv
//------------------------------------------------------------------------------
void	GUIView::renderView( CWbool  inRenderOnly )
{
	CWbool	ret = true;
	if( !inRenderOnly){
		ret = this->onDraw();
	}

//	this->drawInfo();

	//	_O
	if( ret ){
		CWresultCode	result = this->m_glView.getCurrentContext()->randerAndSwapBuffer();
	}
}

//------------------------------------------------------------------------------
//	XN[TCYύX
//------------------------------------------------------------------------------
CWbool			GUIView::changeScreenSize( CWint  inWidth, CWint  inHeight  )
{
	CWresultCode			result;
	GLSContextViewOption	contextOption;

	this->m_glView.getMode( &contextOption );
	if( this->isFullScreen() ){
		this->m_switchingScreenMode = true;
		do{
			//	tF[hAEg
			GLSFader	fader( contextOption.displayID, 30 );
			
			//	؂ւ
			contextOption.width	= inWidth;
			contextOption.height	= inHeight;
			result = this->m_glView.changeMode( &contextOption );
			if( result != CW_NO_ERROR ){
				break;
			}
			
			this->onChangeScreenMode( true );

			//	Yꂸreshape
			this->onReshape();
			
			//	ŕ`悵Ă
			this->renderView();
			
			//	tF[hC

		}while( false );
		this->m_switchingScreenMode = false;
	}
	//	EBhETCYBVFCv͎IɔB
	this->setSize( inWidth, inHeight );
	this->repositionWindow( GVS_WINDOW_POSITION_CENTER_ON_MAIN_SCREEN );

	return result;
}

//------------------------------------------------------------------------------
//	tXN[
//------------------------------------------------------------------------------
CWbool			GUIView::isFullScreen( void )
{
	GLSContextViewOption	contextOption;
	this->m_glView.getMode( &contextOption );
	if( contextOption.viewMode == GLS_VIEW_MODE_WINDOW ){
		return	false;
	}
	else{
		return	true;
	}
}

CWresultCode	GUIView::setFullScreen( CWbool  inTrueIsFullScreenMode, CWbool  inDoRenderFlag )
{
	CWresultCode			result;
	GLSContextViewOption	contextOption;
	static	CWint16			locX, locY;
	static	CWuint32		width, height;

	this->m_glView.getMode( &contextOption );
	contextOption.viewMode = ( inTrueIsFullScreenMode ? GLS_VIEW_MODE_FULL_SCREEN : GLS_VIEW_MODE_WINDOW );

	this->m_switchingScreenMode = true;
	do{
		//	tF[hAEg
		GLSFader	fader( contextOption.displayID, 30 );
		
		this->getCurrentViewSize( &width, &height );
		contextOption.width		= width;
		contextOption.height	= height;

		if( inTrueIsFullScreenMode ){
			//	ʒuL
			this->getLocation( &locX, &locY );
		}

		//	؂ւ
		result = this->m_glView.changeMode( &contextOption );
		if( result != CW_NO_ERROR ){
			break;
		}
		
		if( ! inTrueIsFullScreenMode ){
			//	ʒuĐݒ
			this->setLocation( locX, locY, true );
			this->setSize( width, height );
		}

		this->onChangeScreenMode( inTrueIsFullScreenMode );

		//	Yꂸreshape
		this->onReshape();
		
		if(inDoRenderFlag){
			//	ŕ`悵Ă
			this->renderView();
		}
		
		//	tF[hC

	}while( false );
	this->m_switchingScreenMode = false;
	
	return result;
}

//------------------------------------------------------------------------------
//	Jgr[̃TCY
//------------------------------------------------------------------------------
void	GUIView::getCurrentViewSize( CWuint32*  outWidth, CWuint32*  outHeight )
{
	GLSContext*  context = this->m_glView.getCurrentContext();
	Rect		rectPort;
	CGrafPtr	port = aglGetDrawable( context->getContext());
	GetPortBounds( port, &rectPort );
	*outWidth = rectPort.right - rectPort.left;
	*outHeight = rectPort.bottom - rectPort.top;
}

//------------------------------------------------------------------------------
//	CxgɌĂт܂B
//------------------------------------------------------------------------------
GVSeventResult	GUIView::onWindowEventHandler( const GVSeventView  inEvent )
{
	CWresultCode	result;
	
	switch( inEvent ){
		//	J
		case GVS_EVENT_VIEW_OPEN:
			break;

		//	O
		case GVS_EVENT_VIEW_CLOSE:
			break;

		//	ړ܂
		case GVS_EVENT_VIEW_MOVE: 
			break;

		//	TCY܂
		case GVS_EVENT_VIEW_RESIZE: 
			result = this->m_glView.update();
			this->onReshape();
			break;

		//	Y[{^NbN܂
		case GVS_EVENT_VIEW_ZOOMED:
			break;

		//	ANeBuɂȂ܂
		case GVS_EVENT_VIEW_ACTIVATE:
			break;

		//	fBANeBuɂȂ܂
		case GVS_EVENT_VIEW_DEACTIVATE:
			break;

		//	𑜓x؂ւ܂iʃTCYF[xjtXN[؂ւł͌Ăт܂B
		case GVS_EVENT_VIEW_CHANGE_DISPLAY_MODE:
		
			if( ! this->m_switchingScreenMode ){
				result = this->m_glView.update();
				this->onReshape();
			}
			break;
	};

	return	GVS_EVENT_RESULT_NO;
}

/*
//------------------------------------------------------------------------------
//	}EXCxgɌĂт܂B
//------------------------------------------------------------------------------
GVSeventResult	GUIView::onMouseEventHandler( const GVSeventMouse  inEvent, const GVSMouseEventData*  inEventData )
{
	switch( inEvent ){
		//	{^NbN܂
		case GVS_EVENT_MOUSE_DOWN:
			break;

		//	{^𗣂܂
		case GVS_EVENT_MOUSE_UP:
			break;
		
		//	}EXړ܂
		case GVS_EVENT_MOUSE_MOVED:
			break;

		//	}EXhbO܂
		case GVS_EVENT_MOUSE_DRAGGED:
			break;

		//	zC[𓮂܂
		case GVS_EVENT_MOUSE_WHEEL_MOVED:
			break;
	};

	return	GVS_EVENT_RESULT_NO;
}

//------------------------------------------------------------------------------
//	L[{[hCxgɌĂт܂B
//------------------------------------------------------------------------------
GVSeventResult	GUIView::onKeyboardEventHandler( const GVSeventKeyboard  inEvent, const GVSKeyboardEventData*  inEventData )
{
	switch( inEvent ){
		//	L[܂
		case GVS_EVENT_KEYBORAD_DOWN:
			break;

		//	L[Ă܂
		case GVS_EVENT_KEYBORAD_PRESS:
			break;
		
		//	L[𗣂܂
		case GVS_EVENT_KEYBORAD_UP:
			break;
		
		//	fBt@C[L[𑀍삵܂
		case GVS_EVENT_KEYBORAD_MODIFIERS_CHANGE:
			break;
	};

	return	GVS_EVENT_RESULT_NO;
}

//------------------------------------------------------------------------------
//	^ubgCxgɌĂт܂B
//------------------------------------------------------------------------------
GVSeventResult	GUIView::onTabletEventHandler( const GVSeventTablet  inEvent, const GVSTabletEventData*  inEventData, const GVSTabletProximityEventData*  inProximityEventData )
{
	switch( inEvent ){
		//	^ubg̓͂󂯕t܂
		case GVS_EVENT_TABLET_POINT:
			break;

		//	^ubgfoCXiyjAANeBu܂̓fBANeBuɂȂiڋ߁A܂͗j
		case GVS_EVENT_TABLET_PROXIMITY:
			break;
	};

	return	GVS_EVENT_RESULT_NO;
}
*/