/*//////////////////////////////////////////////////////////////////////////////
*
*	Project:	CodeWorker
*
*	Component:	CodeWorker MacOS X / GameInputService
*	File:		CWGameInputService.h
*
*	Encoding:	Shift-JIS
*	Font:		Osaka-
*
*	Memo:
*				HID@킩̓͏𗘗p邽߂̃T[rXB
*
*	FileVersion:	
*	----------------------------------------------------------------------------
*		version 1.00	2004/02/01	Itinose Shinta
*	----------------------------------------------------------------------------
*
*	Licence:	GNU General Public License
*	----------------------------------------------------------------------------
*	<CodeWorker>
*    Copyright (C) <2004/02/01>  <Itinose Shinta>
*
*    This program is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*	----------------------------------------------------------------------------
*
*///////////////////////////////////////////////////////////////////////////////

#ifndef	CWGameInputService_h
#define	CWGameInputService_h

#include	"CWorkerConfig.h"

//	Carbon
#include <Carbon/Carbon.h>
//	HID
#include <IOKit/hid/IOHIDKeys.h>
#include <IOKit/hid/IOHIDUsageTables.h>

//	CPLUSPLUS
#ifdef	__cplusplus
	#ifdef __MWERKS__
		#pragma		export on
	#endif	__MWERKS__
#endif	__cplusplus
	

////////////////////////////////////////////////////////////////////////////////
//
//	Result Code
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
//	Define
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
//	HID DEVICE REFERENCE
//------------------------------------------------------------------------------
typedef	struct	OPAQUA_STRUCT(_OPAQUA_GISDeviceData)		*GISDeviceRef;	//	s̃f[^

//------------------------------------------------------------------------------
//	HID ELEMENT REFERENCE
//------------------------------------------------------------------------------
typedef	struct	OPAQUA_STRUCT(_OPAQUA_GISElementData)		*GISElementRef;	//	s̃f[^

//------------------------------------------------------------------------------
//	HID DEVICE INFORMATION HEADER
//------------------------------------------------------------------------------
struct	GISDeviceHeader{
    CWint32		vendorID;				// id for device vendor, unique across all devices
    CWint32		productID;				// id for particular product, unique across all of a vendors devices
    CWint32		version;				// version of product
    CWint32		deviceUsage;			// usage page from IOUSBHID Parser.h which defines general usage
    CWint32		deviceUsagePage;		// usage within above page from IOUSBHID Parser.h which defines specific usage
};
typedef	struct	GISDeviceHeader		GISDeviceHeader;

//------------------------------------------------------------------------------
//	HID DEVICE INFORMATION
//------------------------------------------------------------------------------
struct	GISDeviceInfo{
	CWint32			deviceNumber;		// ނ̃foCX݂Ƃ̃foCXʔԍ蓖Ă܂BSfoCXL鎞̎ʂɗp܂B
    CWint32			locationID;			// long representing location in USB (or other I/O) chain which device is pluged into, can identify specific device on machine
    CFStringRef		transport;			// device transport
    CFStringRef		manufacturerName;	// name of manufacturer( vender name )
    CFStringRef		productName;		// name of product
    CFStringRef		serialNumber;		// serial number of specific product, can be assumed unique across specific product or specific vendor (not used often)

    struct{
	    CWint32	totalCount;				// number of total elements (should be total of all elements on device including collections) (calculated, not reported by device)

		CWint32 countTypeFeatures;		// number of elements of type kIOHIDElementTypeFeature
		CWint32 countTypeInputs;		// number of elements of type kIOHIDElementTypeInput_Misc or kIOHIDElementTypeInput_Button or kIOHIDElementTypeInput_Axis or kIOHIDElementTypeInput_ScanCodes
		CWint32 countTypeOutputs;		// number of elements of type kIOHIDElementTypeOutput
		CWint32 countTypeCollections;	// number of elements of type kIOHIDElementTypeCollection
		CWint32 countTypeUnknow;		// number of elements of type unknow

	    CWint32 countAxis;			// number of axis (calculated, not reported by device)
	    CWint32 countButtons;		// number of buttons (calculated, not reported by device)
	    CWint32 countHats;			// number of hat switches (calculated, not reported by device)
	    CWint32 countSliders;		// number of sliders (calculated, not reported by device)
	    CWint32 countDials;			// number of dials (calculated, not reported by device)
	    CWint32 countWheels;		// number of wheels (calculated, not reported by device)
	    
	    CWint32 countUnknow;		// number of unknow (calculated, not reported by device)
    } elementCount;
};
typedef	struct	GISDeviceInfo		GISDeviceInfo;

//------------------------------------------------------------------------------
//	HID ELEMENT INFORMATION HEADER
//------------------------------------------------------------------------------
struct	GISElementHeader{
    IOHIDElementType		elementType;			// the element type
    CWint32 				elementUsagePage;		// usage page from IOUSBHIDParser.h which defines general usage
    CWint32 				elementUsage;			// usage within above page from IOUSBHIDParser.h which defines specific usage
    IOHIDElementCookie		elementCookie;			// unique value (within device of specific vendorID and productID) which identifies element, will NOT change
};
typedef	struct	GISElementHeader	GISElementHeader;

//------------------------------------------------------------------------------
//	HID ELEMENT INFORMATION
//------------------------------------------------------------------------------
struct	GISElementInfo{
	GISDeviceHeader*	deviceHeader;	//	foCX
	GISDeviceInfo*		deviceInfo;		//	foCX

    CFStringRef	elementName;			//	name of element
    CWbool		trueIsLookupError;		//	O擾ɎsƂtrueĂB

    CWint32 	valueMin;				//	reported min value possible
    CWint32 	valueDefault;			//	reported center value possible		ĂȂԂ̒lB
    CWint32 	valueMax;				//	reported max value possible
    CWint32 	scaledMin;				//	reported scaled min value possible
    CWint32 	scaledMax;				//	reported scaled max value possible
    CWint32 	elementSize;			//	size in bits of data return from element
    CWbool		trueIsRelative;			//	are reports relative to last report (deltas)
    CWbool		trueIsWrapping;			//	does element wrap around (one value higher than max is min)
    CWbool		trueIsNonLinear;		//	are the values reported non-linear relative to element movement
    CWbool		trueIsPreferredState;	//	does element have a preferred state (such as a button)
    CWbool		trueIsNullState;		//	does element have null state
    CWint32 	unitCount;				//	units value is reported in (not used very often)
    CWint32		unitExponent;			//	exponent for units (also not used very often)

    CWint32		calibrateValueMin; 		// min returned value
    CWint32 	calibrateValueMax; 		// max returned value (calibrate call)

	CWint32 	depth;					//	ċA̐[
};
typedef	struct	GISElementInfo		GISElementInfo;

//------------------------------------------------------------------------------
//	ELEMENT TYPE 
//		Gǧ̃tB^OɎgpB
//------------------------------------------------------------------------------
enum{
	GIS_ELEMENT_TYPE_INPUT_MISC			= 1 << 1,		//	kIOHIDElementTypeInput_Misc
	GIS_ELEMENT_TYPE_INPUT_BUTTON		= 1 << 2,		//	kIOHIDElementTypeInput_Button
	GIS_ELEMENT_TYPE_INPUT_AXIS			= 1 << 3,		//	kIOHIDElementTypeInput_Axis
	GIS_ELEMENT_TYPE_INPUT_SCANCODES	= 1 << 4,		//	kIOHIDElementTypeInput_ScanCodes
	GIS_ELEMENT_TYPE_OUTPUT				= 1 << 5,		//	kIOHIDElementTypeOutput

	GIS_ELEMENT_TYPE_FEATURE			= 1 << 6,		//	kIOHIDElementTypeFeature
	GIS_ELEMENT_TYPE_COLLECTION			= 1 << 7,		//	kIOHIDElementTypeCollection
	
	GIS_ELEMENT_TYPE_INPUT				= GIS_ELEMENT_TYPE_INPUT_MISC | 
											GIS_ELEMENT_TYPE_INPUT_BUTTON | 
											GIS_ELEMENT_TYPE_INPUT_AXIS | 
											GIS_ELEMENT_TYPE_INPUT_SCANCODES,
											
	GIS_ELEMENT_TYPE_IN_OUT				= GIS_ELEMENT_TYPE_INPUT | 
											GIS_ELEMENT_TYPE_OUTPUT |
											GIS_ELEMENT_TYPE_FEATURE,
											
	GIS_ELEMENT_TYPE_ALL				= GIS_ELEMENT_TYPE_IN_OUT | GIS_ELEMENT_TYPE_COLLECTION,
};
typedef CWtype		GISElementType;

//------------------------------------------------------------------------------
//	DEVICE EVENT HANDLING 
//------------------------------------------------------------------------------
//	device activate type
//		GISDeviceEventFuncR[obN֐EventʂɎgpB
enum{
	GIS_DEVICE_ACTIVATE_PLUGIN,		//	foCXڑ
	GIS_DEVICE_ACTIVATE_REMOVE,		//	foCX𔲂
};
typedef	CWtype	GISDeviceActivateType;

//	device event handler
//		foCXEvent󂯎R[obN֐B
//
//		F GISDeviceRef				inDevice		Cxg̔foCX
//		F GISDeviceActivateType	inType			Cxg̎
//		F void*					inUserData		GISInstallDeviceActivate()Ŏw肵UserData
//
typedef	void	(*GISDeviceActivateFunc)( GISDeviceRef  inDevice, GISDeviceActivateType  inType, void*  inUserData );



////////////////////////////////////////////////////////////////////////////////
//
//	Interface
//
////////////////////////////////////////////////////////////////////////////////
//	HID CONTROL
//------------------------------------------------------------------------------
//	HID CONTROL
//------------------------------------------------------------------------------
//	Initialize
//		ď\BAďƑSĂ GISDeviceRef, GISElementRef͖ƂȂAS~ŵŒӂ邱ƁB
//
//		ߒlF CWresultCode			G[R[h
CW_EXTERN	CWresultCode	GISInitialize( void );

//	Finalize
CW_EXTERN	void			GISFinalize( void );

//------------------------------------------------------------------------------
//	DEVICE
//------------------------------------------------------------------------------
//	Begin get devices
//		foCX̎擾Jn
CW_EXTERN	void			GISBeginGetDevices( void );

//	Get device reference
//		foCXԂɎ擾BSēǂݎƁANULLԂB
//		GISDeviceRef̓[mۂȂ̂ŁA폜̕Kv͗L܂B
//
//		ߒlF GISDeviceRef						foCX
CW_EXTERN	GISDeviceRef	GISGetNextDeviceReference( void );

//------------------------------------------------------------------------------
//	ELEMENT
//------------------------------------------------------------------------------
//	Begin get elements
//		Gg̎擾Jn
CW_EXTERN	void			GISBeginGetElements( const GISDeviceRef  inDeviceRef );

//	Get element reference
//		GgԂɎ擾BSēǂݎƁANULLԂB
//		GISElementRef̓[mۂȂ̂ŁA폜̕Kv͗L܂B
//
//		F GISElementType	inElementType		tB^[wBw肵^CṽGĝݎQƂB
//		ߒlF GISElementRef						Gg
CW_EXTERN	GISElementRef	GISGetNextElementReference( const GISElementType  inElementType );

//------------------------------------------------------------------------------
//	INFORMATION
//------------------------------------------------------------------------------
//	Get device information
//		F GISDeviceRef		inDeviceRef		QƂfoCX
//		ߒlF GISDeviceHeader*					Header̃|C^
CW_EXTERN	GISDeviceHeader*	GISGetDeviceHeader( const GISDeviceRef  inDeviceRef );

//	Get device header
//		F GISElementRef	inElementRef	Gg
//		ߒlF GISDeviceInfo*					informatioñ|C^
CW_EXTERN	GISDeviceInfo*		GISGetDeviceInformation( const GISDeviceRef  inDeviceRef );

//	Get element information
//		F GISElementRef	inElementRef	Gg
//		ߒlF GISElementHeader*				Header̃|C^
CW_EXTERN	GISElementHeader*	GISGetElementHeader( const GISElementRef  inElementRef );

//	Get element header
//		F GISElementRef	inElementRef	Gg
//		ߒlF GISElementInfo*					informatioñ|C^
CW_EXTERN	GISElementInfo*		GISGetElementInformation( const GISElementRef  inElementRef );

//	Get deviceRef
//		GgfoCX擾B
//		F GISElementRef	inElementRef	Gg
//		ߒlF GISDeviceRef						Gg̏L҂łfoCX
CW_EXTERN	GISDeviceRef		GISGetDeviceRef( const GISElementRef  inElementRef );

//	Get device count
//		ߒlF CWuint32						HIDfoCX̑
CW_EXTERN	CWuint32		GISGetDeviceCount( void );

//	Set device user data
//		F GISDeviceRef	inDeviceRef		foCX
//		F void*		inUserData		[U[f[^
//		ߒlF CWresultCode					G[R[h
CW_EXTERN	CWresultCode	GISSetDeviceUserData( const GISDeviceRef  inDeviceRef, const void*  inUserData );

//	Get device user data
//		F GISDeviceRef  inDeviceRef		foCX
//		ߒlF void*							[U[f[^
CW_EXTERN	void*			GISGetDeviceUserData( const GISDeviceRef  inDeviceRef );

//	Set element user data
//		F GISElementRef	inElementRef	Gg
//		ߒlF void*							[U[f[^
//		ߒlF CWresultCode						G[R[h	
CW_EXTERN	CWresultCode	GISSetElementUserData( const GISElementRef  inElementRef, const void*  inUserData );

//	Get element user data
//		F GISElementRef	inElementRef	Gg
//		ߒlF void*							[U[f[^
CW_EXTERN	void*			GISGetElementUserData( const GISElementRef  inElementRef );

//------------------------------------------------------------------------------
//	ELEMENT STATUS
//------------------------------------------------------------------------------
//	Get element value
//		̒lQƂB
//		F GISElementRef	inElementRef	Gg
//		ߒlF CWint32							l
CW_EXTERN	CWint32	GISGetElementValue( const GISElementRef  inElementRef );

//	Get element Calibrate value 
//		Lu[g␳{lԂB
//		F GISElementRef	inElementRef	Gg
//		F CWint32  		inValue			GISGetElementValue()̒lnĂB
//		ߒlF CWint32							l
CW_EXTERN	CWint32	GISGetElementCalibrateValue( const GISElementRef  inElementRef, const CWint32  inValue );

//------------------------------------------------------------------------------
//	ACTIVATE HANDLING ( DEVICE  MANAGEMENT )
//------------------------------------------------------------------------------
//	install callback device activate handler
//		foCXڑA邢͔ꂽƂɌĂяoR[obNB
//		R[obN͕CXg[oB
//
//		F GISDeviceActivateFunc	inFunc		CXg[R[obN֐̃|C^
//		F void*  					inUserData	R[obNɈnf[^
//		ߒlF CWresultCode							G[R[h
CW_EXTERN	CWresultCode	GISInstallDeviceActivate( const GISDeviceActivateFunc  inFunc, void*  inUserData );

//	remove callback device Activate handler
//		F GISDeviceActivateFunc	inFunc	菜R[obN֐̃|C^
CW_EXTERN	void			GISRemoveDeviceActivate( const GISDeviceActivateFunc  inFunc );

//	get callback device Activate handler
//		F CWuint32  				inNumber	CXg[ԍ
//		F GISDeviceActivateFunc	inFunc		R[obN֐̃|C^
//		F void*  					inUserData	R[obNɈnf[^
//		ߒlF CWbool								true = LAfalse = 
CW_EXTERN	CWbool			GISGetDeviceActivate( const CWuint  inNumber, GISDeviceActivateFunc*  outFunc, void**  outUserData );

//------------------------------------------------------------------------------
//	EVENT BUFFER ( ELEMENT  STATE )
//		EEventHandlinǵACarbonEventȂǉEvent[vsĂȂƌʂ܂B
//
//		EApplicationactivateAdeactivateCxgŁAfoCX̃obt@O~TOĂ܂B
//		@]InputSprocketɗL悤ȁAJnEIEResumeƂ֐͗L܂B
//
//		@ƂȂ̂ApplicationANeBuł͖łB
//		@Q[sAeLXgGfB^ŕłlbgT[tBył݂肵ĂƂ
//		@HID͒lԂAobt@O܂B
//		@backgroundɗLƂłAQ[𑀍łĂ܂̂łB
//
//		@ɂ́AApplicationdeactivateCxgȍ~ANCAgR[hHID𖳎悤ɂA
//		@activateCxgŃCxgBufferFLASHāAHIDs悤ɂ܂B
//		@Q[|[YԂɂꍇ́AbackgroundESCoȂ悤ӂ܂B
//
//------------------------------------------------------------------------------
//	Start Device event buffer
//		Cxgobt@[̎gpJnB
//		KvȃfoCXɊJnĂB
//
//		F GISDeviceRef		inDeviceRef				foCX
//		ߒlF CWresultCode								G[R[h
CW_EXTERN	CWresultCode	GISStartDeviceEvent( const GISDeviceRef  inDeviceRef );

//	Stop Device event buffer
//		Cxgobt@[̎gp~
//
//		F GISDeviceRef		inDeviceRef				foCX
CW_EXTERN	void			GISStopDeviceEvent( const GISDeviceRef  inDeviceRef );

//	Start Element Event buffer
//		KvȃGgEventobt@[̎gpJnĂB
//
//		F GISElementRef	inElementRef	Gg
//		ߒlF CWresultCode						G[R[h
CW_EXTERN	CWresultCode	GISStartElementEvent( const GISElementRef  inElementRef );

//	stop Element Event buffer
//		F GISElementRef	inElementRef	Gg
CW_EXTERN	void			GISStopElementEvent( const GISElementRef  inElementRef );

//	get event value
//		Queue͏Ԃ擾BQueueo߁AGISGetElementValue()ƈႢAߋeRȂ擾B
//		[U[Ń{^悤ȃQ[ł̎肱ڂhƂoB
//		܂A{^ςȂ̏Ԃ̎łAuԂo邱ƂoBςȂԂ̌oł͍鎞ɖ𗧂B
//		L[_Ei{^jAL[Abvi{^jƂQxCxgE悤łB
//
//		F GISDeviceRef		inDeviceRef			foCX
//		F CWbool*			outTrueIsGetEvent	Event擾łǂBtrue = 擾Afalse = 擾Ȃ
//		F GISElementRef*	outElement			Event̗LGg
//		F CWint32*			outValue			擾l
//		F AbsoluteTime*	outTime				擾
//		ߒlF CWresultCode							G[R[h
CW_EXTERN	CWresultCode	GISGetElementEventValue( const GISDeviceRef  inDeviceRef, CWbool*  outTrueIsGetEvent, 
														GISElementRef*  outElement, CWint32*  outValue, AbsoluteTime*  outTime /*= NULL*/ );

//	flash event
//		SẴfoCXQueueɂB
CW_EXTERN	void	GISFlashDeviceEvent( void );

//------------------------------------------------------------------------------
//	CALLBACK / DEVICE STATUS
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
//	CONFIGURATION
//
//		EIGg̕ۑƍČ
//			{Cuł͕ۑEČ@\͗L܂B
//			[U[IGgANɍČ悤ɂĂB
//			EۑAGgʂޓɏ݂܂BKvȂ̂́AGISDeviceHeaderƁAGISElementHeader݂̂łB
//			EČ́AGISDeviceHeaderAGISElementHeaderɃGgT܂BȂꍇlKvL܂B
//
//------------------------------------------------------------------------------
//	Search active element
//
//		͔̗LGg擾B
//		L[RtBOœ̓foCX̑IɎgpB
//		ɂĕ̃Ggo邽߁AzŎ󂯎B
//
//		ӁB
//		̃Gg鎖B
//		@Ⴆ΁AWN[CENTERhCo[CXg[[U[ł́AL[{[hƂQELEMENToB
//		@}EXȏ̂ƂA邩܂i
//			i̗FAL[ "A"ELEMENTƁAusage = -1 ̂QELEMENTɁAɒlZbgB
//			@ʓIɁA^b`ŁÅ֐ł͓ELEMENToĂ܂Bj
//
//		element Usege = -1 ELEMENT鎖LB
//		@-1ELEMENT́AreserveƂƂɂȂĂB͗\łȂ̂ŁAׂELEMENTłB
//		@[U[ɂẮA\ĂȂGgԂĂ鎖LB
//
//		WN[CENTERhCo[ł́AGISGetElementValue()œlŃL[R[h𔻕ʂ邱ƂłB
//		@ASCIIR[h𓾂ƂƂŁA͑ϕ֗ł邪A
//		@usage@-1@ł邱ƂƁAɁ@-1@ELEMENT݂Ă邽߁AזȃGgłB
//
//		F GISElementType	inElementType			擾Gg̃^Cv
//		F CWint			inTimeOutTickCount		^CAEgƂȂ鎞ԁi1/60bPʂŎw肷邱Ɓj
//		F GISElementRef*	outElementRefArray		擾Gg̔z
//		F CWsize			inArrayCount			擾z̔z
//		ߒlF CWresultCode								G[R[h
CW_EXTERN	CWbool	GISSearchActiveElement( const GISElementType  inElementType, const CWint  inTimeOutTickCount, GISElementRef*  outElementRefArray, const CWsize  inArrayCount );

//------------------------------------------------------------------------------
//	Utility
//------------------------------------------------------------------------------
//	Get Element type name
//		IOHIDElementTypê̖ԂBGUI\ł̗pzB
//
//		F IOHIDElementType	inTyep			IOHIDElementType萔
//		F CWint8*			outName			C󂯎obt@
//		F CWsize			inSize			obt@̃oCg
//		ߒlF CWbool							G[̗Li true = A false = s j
CW_EXTERN	CWbool	GetElementTypeName( const IOHIDElementType  inTyep, CWint8*  outName, const CWsize  inSize );

//	C String <-- CFStringRef
//		CFStringRefCɕϊB
//
//		F CFStringRef		inStringRef			CFStringRef
//		F CWint8*			outCString			C󂯎obt@
//		F CWsize			outCStringSize		obt@̃oCg
//		ߒlF CWbool								G[̗Li true = A false = s j
CW_EXTERN	CWbool	GISGetCString( const CFStringRef  inStringRef, CWint8*  outCString, const CWsize  outCStringSize );

//------------------------------------------------------------------------------
//	DEBUGGING
//------------------------------------------------------------------------------
//	output to consol device information
//		foCX̑SR\[֏o͂B
//		
//		F GISDeviceRef		inDeviceRef				foCX
//		F CWbool			inTrueIsShowElements	foCXLGgɂĂo͂邩ǂ̎wi true = o͂A false = o͂Ȃ j
CW_EXTERN	void	GISOutputDeviceInformation( const GISDeviceRef  inDeviceRef /*= NULL*/, const CWbool  inTrueIsShowElements /*= true*/ );

//	output to consol element information
//		Gg̑SR\[֏o͂B
//
//		F GISElementRef	inElementRef			Gg
CW_EXTERN	void	GISOutputElementInformation( const GISElementRef  inElementRef );

#ifdef __cplusplus
	#ifdef __MWERKS__
		#pragma		export off
	#endif __MWERKS__
#endif __cplusplus

#endif	//CWGameInputService_h