/*//////////////////////////////////////////////////////////////////////////////
*
*	Project:	CodeWorker
*
*	Component:	CodeWorker MacOS X / PreferenceService
*	File:		CWPreferenceService.cpp
*
*	Encoding:	Shift-JIS
*	Font:		Osaka-
*
*	Memo:
*
*	FileVersion:	
*	----------------------------------------------------------------------------
*		version 1.00	2004/03/29	Itinose Shinta
*	----------------------------------------------------------------------------
*
*	Licence:	GNU General Public License
*	----------------------------------------------------------------------------
*	<CodeWorker>
*	Copyright (C) <2004/02/01>  <Itinose Shinta>
*
*	This program is free software; you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation; either version 2 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*	----------------------------------------------------------------------------
*
*///////////////////////////////////////////////////////////////////////////////

#include	<Carbon/Carbon.h>
#include	"CWorkerConfig.h"
#include	"CWPreferenceService.h"

namespace	code_worker{

////////////////////////////////////////////////////////////////////////////////
//
//	FUNCTION
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
//	write preference data
//------------------------------------------------------------------------------
CWbool			PSWriteInt( const CFStringRef  inKeyString, const CWint  inValue )
{
    CFNumberRef value;

    value = CFNumberCreate( NULL, kCFNumberIntType, &inValue ); 
	CFPreferencesSetAppValue( inKeyString, value, kCFPreferencesCurrentApplication );
    CFRelease(value);

	return true;
}

CWbool			PSWriteFloat( const CFStringRef  inKeyString, const CWfloat32  inValue )
{
    CFNumberRef value;

    value = CFNumberCreate( NULL, kCFNumberFloatType, &inValue ); 
	CFPreferencesSetAppValue( inKeyString, value, kCFPreferencesCurrentApplication );
    CFRelease(value);

	return true;
}

CWbool			PSWriteDouble( const CFStringRef  inKeyString, const CWfloat64  inValue )
{
    CFNumberRef value;

    value = CFNumberCreate( NULL, kCFNumberDoubleType, &inValue ); 
	CFPreferencesSetAppValue( inKeyString, value, kCFPreferencesCurrentApplication );
    CFRelease(value);

	return true;
}

CWbool			PSWriteString( const CFStringRef  inKeyString, const CFStringRef  inValue )
{
	CFPreferencesSetAppValue( inKeyString, inValue, kCFPreferencesCurrentApplication );

	return true;
}

CWbool			PSWriteBinary( const CFStringRef  inKeyString, const CWuint8*  inData, const CWsize  inSize )
{
    CFDataRef 	dataBuffe = CFDataCreate( kCFAllocatorDefault, inData, inSize ); 
	CFPreferencesSetAppValue( inKeyString, dataBuffe, kCFPreferencesCurrentApplication );
    CFRelease(dataBuffe);

	return true;
}

//------------------------------------------------------------------------------
//	read preference data
//------------------------------------------------------------------------------
CWint			PSCopyInt( const CFStringRef  inKeyString )
{
	CWint			intValue;
	Boolean		exist;

    intValue = CFPreferencesGetAppIntegerValue( inKeyString, kCFPreferencesCurrentApplication, &exist );   

	if( ! exist ){
		intValue = 0;
	}
	return intValue;
}

CWfloat32		PSCopyFloat( const CFStringRef  inKeyString )
{
	CWfloat32	intValue;
    CFNumberRef value;

    value = (CFNumberRef)CFPreferencesCopyAppValue( inKeyString, kCFPreferencesCurrentApplication );   
	if( ! CFNumberGetValue( value, kCFNumberFloatType, &intValue ) ){
		intValue = 0;
	}
	CFRelease( value );

	return intValue;
}

CWfloat64		PSCopyDouble( const CFStringRef  inKeyString )
{
	CWfloat64		intValue;
    CFNumberRef value;

    value = (CFNumberRef)CFPreferencesCopyAppValue( inKeyString, kCFPreferencesCurrentApplication );   
	if( ! CFNumberGetValue( value, kCFNumberDoubleType, &intValue ) ){
		intValue = 0;
	}
	CFRelease( value );

	return intValue;
}

CFStringRef		PSCopyString( const CFStringRef  inKeyString )
{
	CFStringRef  valString;

    valString = (CFStringRef)CFPreferencesCopyAppValue( inKeyString, kCFPreferencesCurrentApplication );   

	return valString;
}

CWsize			PSGetBinarySize( const CFStringRef  inKeyString )
{
	CWsize		retSize;
    CFDataRef 	dataBuffe;

    dataBuffe = (CFDataRef)CFPreferencesCopyAppValue( inKeyString, kCFPreferencesCurrentApplication );   
	retSize = CFDataGetLength( dataBuffe );
	CFRelease( dataBuffe );

	return retSize;
}
CWbool			PSCopyBinary( const CFStringRef  inKeyString, CWuint8*  ioData, const CWsize  inSize )
{
	CWbool				ret = true;
	const CWuint8*		retBuffer;
    CFDataRef 			dataBuffe;

    dataBuffe = (CFDataRef)CFPreferencesCopyAppValue( inKeyString, kCFPreferencesCurrentApplication );   
	retBuffer = CFDataGetBytePtr( dataBuffe );
	if( retBuffer != NULL ){
		ret = false;
	}
	else{
		memcpy( ioData, retBuffer, inSize );
	}
	CFRelease( dataBuffe );

	return ret;
}


//------------------------------------------------------------------------------
//	read preference data / syncro
//------------------------------------------------------------------------------
void				PSSynchronize( void )
{
    (void)CFPreferencesAppSynchronize( kCFPreferencesCurrentApplication );
}


//------------------------------------------------------------------------------
//	get identity
//------------------------------------------------------------------------------
CFStringRef		PSGetIdentity( void )
{
	CFStringRef		identity	= NULL;
	CFBundleRef		appBundle	= NULL;

	/*
	*	get propertyList info
	*/
	//	sAvP[Ṽoh
	appBundle = CFBundleGetMainBundle();
	if( appBundle != NULL ){
		//	propertyList擾
		identity = (CFStringRef) CFBundleGetValueForInfoDictionaryKey( appBundle, CFSTR("CFBundleIdentifier") );
	}
	return identity;
}


}	//	code_worker